#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1beta1.StatefulSet import StatefulSet


class K8sStatefulSet(K8sObject):
    """
    https://kubernetes.io/docs/api-reference/apps/v1beta1/definitions/#_v1beta1_statefulset
    """

    def __init__(self, config=None, name=None):
        super(K8sStatefulSet, self).__init__(
            config=config,
            name=name,
            obj_type='StatefulSet'
        )

    # -------------------------------------------------------------------------------------  override

    def get(self):
        self.model = StatefulSet(self.get_model())
        return self

    def create(self):
        super(K8sStatefulSet, self).create()
        self.get()
        return self

    def update(self):
        super(K8sStatefulSet, self).update()
        self.get()
        return self

    def list(self, pattern=None):
        ls = super(K8sStatefulSet, self).list()
        ssets = list(map(lambda x: StatefulSet(x), ls))
        if pattern is not None:
            ssets = list(filter(lambda x: pattern in x.name, ssets))
        k8s = []
        for x in ssets:
            j = K8sStatefulSet(config=self.config, name=x.name)
            j.model = x
            k8s.append(j)
        return k8s



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1alpha1.PetSet import PetSet
from kubernetes.utils import is_reachable, sanitize_version_number


class K8sPetSet(K8sObject):
    """
    Starting in Kubernetes version 1.5, PetSet has been refactored into StatefulSet.

    To continue to use PetSets in Kubernetes 1.5 or higher, you must migrate your existing PetSets to StatefulSets.
    See https://kubernetes.io/docs/tasks/manage-stateful-set/upgrade-pet-set-to-stateful-set/
    """

    def __init__(self, config=None, name=None):

        if config and is_reachable(config):
            temp = K8sObject(config=config, obj_type='Pod', name='temp')
            v = temp.server_version()
            major_vernum = sanitize_version_number(v['major'])
            minor_vernum = sanitize_version_number(v['minor'])
            if major_vernum <= 1 and minor_vernum < 4:
                raise NotImplementedError('PetSets exist only on Kubernetes == 1.4.x.')
            if major_vernum >= 1 and minor_vernum >= 5:
                raise NotImplementedError('PetSets were refactored into StatefulSets on Kubernetes >= 1.5.x.')

        super(K8sPetSet, self).__init__(
            config=config,
            name=name,
            obj_type='PetSet'
        )

    # -------------------------------------------------------------------------------------  override

    def get(self):
        self.model = PetSet(self.get_model())
        return self

    def create(self):
        super(K8sPetSet, self).create()
        self.get()
        return self

    def update(self):
        super(K8sPetSet, self).update()
        self.get()
        return self

    def list(self, pattern=None):
        ls = super(K8sPetSet, self).list()
        pets = list(map(lambda x: PetSet(x), ls))
        if pattern is not None:
            pets = list(filter(lambda x: pattern in x.name, pets))
        k8s = []
        for x in pets:
            j = K8sPetSet(config=self.config, name=x.name)
            j.model = x
            k8s.append(j)
        return k8s



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#


class RollbackConfig(object):
    """
    http://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_rollbackconfig
    """

    def __init__(self, model=None):
        super(RollbackConfig, self).__init__()

        self._revision = 0

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'revision' in model:
            self.revision = model['revision']

    # ------------------------------------------------------------------------------------- revision

    @property
    def revision(self):
        return self._revision

    @revision.setter
    def revision(self, rev=None):
        if not isinstance(rev, int):
            raise SyntaxError('RollbackConfig: revision: [ {} ] is invalid.'.format(rev))
        self._revision = rev

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.revision is not None:
            data['revision'] = self.revision
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, is_valid_list


class LabelSelectorRequirement(object):
    """
    http://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_labelselectorrequirement
    """

    def __init__(self, model=None):
        super(LabelSelectorRequirement, self).__init__()

        self._key = None
        self._operator = None
        self._values = []

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'key' in model:
            self.key = model['key']
        if 'operator' in model:
            self.operator = model['operator']
        if 'values' in model:
            self.values = model['values']

    # ------------------------------------------------------------------------------------- key

    @property
    def key(self):
        return self._key

    @key.setter
    def key(self, key=None):
        if not is_valid_string(key):
            raise SyntaxError('LabelSelectorRequirements: key: [ {} ] is invalid.'.format(key))
        self._key = key

    # ------------------------------------------------------------------------------------- operator

    @property
    def operator(self):
        return self._operator

    @operator.setter
    def operator(self, op=None):
        if not is_valid_string(op):
            raise SyntaxError('LabelSelectorRequirements: operator: [ {} ] is invalid.'.format(op))
        self._operator = op

    # ------------------------------------------------------------------------------------- values

    @property
    def values(self):
        return self._values

    @values.setter
    def values(self, v=None):
        if not is_valid_list(v, str):
            raise SyntaxError('LabelSelectorRequirements: values: [ {} ] is invalid.'.format(v))
        self._values = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.key is not None:
            data['key'] = self.key
        if self.operator is not None:
            data['operator'] = self.operator
        if self.values is not None:
            data['values'] = self.values
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class LocalObjectReference(object):

    def __init__(self, model=None):
        super(LocalObjectReference, self).__init__()

        self._name = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'name' in model:
            self.name = model['name']

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, n=None):
        if not is_valid_string(n):
            raise SyntaxError('LocalObjectReference: name: [ {} ] is invalid.'.format(n))
        self._name = n

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.name is not None:
            data['name'] = self.name
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class SELinuxOptions(object):
    """
    https://kubernetes.io/docs/api-reference/v1.6/#selinuxoptions-v1-core
    """

    def __init__(self, model=None):
        super(SELinuxOptions, self).__init__()

        self._level = None
        self._role = None
        self._type = None
        self._user = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'level' in model:
            self.level = model['level']
        if 'role' in model:
            self.role = model['role']
        if 'type' in model:
            self.type = model['type']
        if 'user' in model:
            self.user = model['user']

    # ------------------------------------------------------------------------------------- level

    @property
    def level(self):
        return self._level

    @level.setter
    def level(self, l=None):
        if not is_valid_string(l):
            raise SyntaxError('SELinuxOptions: level: [ {} ] is invalid.'.format(l))
        self._level = l

    # ------------------------------------------------------------------------------------- role

    @property
    def role(self):
        return self._role

    @role.setter
    def role(self, r=None):
        if not is_valid_string(r):
            raise SyntaxError('SELinuxOptions: role: [ {} ] is invalid.'.format(r))
        self._role = r

    # ------------------------------------------------------------------------------------- type

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, t=None):
        if not is_valid_string(t):
            raise SyntaxError('SELinuxOptions: type: [ {} ] is invalid.'.format(t))
        self._level = t

    # ------------------------------------------------------------------------------------- user

    @property
    def user(self):
        return self._user

    @user.setter
    def user(self, u=None):
        if not is_valid_string(u):
            raise SyntaxError('SELinuxOptions: user: [ {} ] is invalid.'.format(u))
        self._user = u

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.level is not None:
            data['level'] = self.level
        if self.role is not None:
            data['role'] = self.role
        if self.type is not None:
            data['type'] = self.type
        if self.user is not None:
            data['user'] = self.user
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#


class StatefulSetStatus(object):

    def __init__(self, model=None):
        super(StatefulSetStatus, self).__init__()

        self._observed_generation = None
        self._replicas = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'observedGeneration' in model:
            self.observed_generation = model['observedGeneration']
        if 'replicas' in model:
            self.replicas = model['replicas']

    # ------------------------------------------------------------------------------------- observedGeneration

    @property
    def observed_generation(self):
        return self._observed_generation

    @observed_generation.setter
    def observed_generation(self, og=None):
        if not isinstance(og, int):
            raise SyntaxError("StatefulSetStatus: observed_generation: [ {} ] is invalid.".format(og))
        self._observed_generation = og

    # ------------------------------------------------------------------------------------- replicas

    @property
    def replicas(self):
        return self._replicas

    @replicas.setter
    def replicas(self, r=None):
        if not isinstance(r, int):
            raise SyntaxError('StatefulSetStatus: replicas: [ {} ] is invalid.'.format(r))
        self._replicas = r

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.observed_generation is not None:
            data['observedGeneration'] = self.observed_generation
        if self.replicas is not None:
            data['replicas'] = self.replicas
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_date_time


class HorizontalPodAutoscalerStatus(object):
    """
    https://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_horizontalpodautoscalerstatus
    """

    def __init__(self, model=None):
        super(HorizontalPodAutoscalerStatus, self).__init__()

        self._observed_generation = None
        self._last_scale_time = None
        self._current_replicas = None
        self._desired_replicas = None
        self._current_cpu_utilization_percentage = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'observedGeneration' in model:
            self.observed_generation = model['observedGeneration']
        if 'lastScaleTime' in model:
            self.last_scale_time = model['lastScaleTime']
        if 'currentReplicas' in model:
            self.current_replicas = model['currentReplicas']
        if 'desiredReplicas' in model:
            self.desired_replicas = model['desiredReplicas']
        if 'currentCpuUtilizationPercentage' in model:
            self.current_cpu_utilization_percentage = model['currentCpuUtilizationPercentage']

    # ------------------------------------------------------------------------------------- observedGeneration

    @property
    def observed_generation(self):
        return self._observed_generation

    @observed_generation.setter
    def observed_generation(self, og=None):
        if not isinstance(og, int):
            raise SyntaxError('HorizontalPodAutoscalerStatus: observed_generation: [ {} ] is invalid.'.format(og))
        self._observed_generation = og

    # ------------------------------------------------------------------------------------- lastScaleTime

    @property
    def last_scale_time(self):
        return self._last_scale_time

    @last_scale_time.setter
    def last_scale_time(self, t=None):
        if not is_valid_date_time(t):
            raise SyntaxError('HorizontalPodAutoscaler: last_scale_time: [ {} ] is invalid.'.format(t))
        self._last_scale_time = t

    # ------------------------------------------------------------------------------------- currentReplicas

    @property
    def current_replicas(self):
        return self._current_replicas

    @current_replicas.setter
    def current_replicas(self, r=None):
        if not isinstance(r, int):
            raise SyntaxError('HorizontalPodAutoscaler: current_replicas: [ {} ] is invalid.'.format(r))
        self._current_replicas = r

    # ------------------------------------------------------------------------------------- desiredReplicas

    @property
    def desired_replicas(self):
        return self._desired_replicas

    @desired_replicas.setter
    def desired_replicas(self, r=None):
        if not isinstance(r, int):
            raise SyntaxError('HorizontalPodAutoscaler: desired_replicas: [ {} ] is invalid.'.format(r))
        self._desired_replicas = r

    # ------------------------------------------------------------------------------------- currentCPU

    @property
    def current_cpu_utilization_percentage(self):
        return self._current_cpu_utilization_percentage

    @current_cpu_utilization_percentage.setter
    def current_cpu_utilization_percentage(self, pct=None):
        if not isinstance(pct, int):
            raise SyntaxError('HorizontalPodAutoscaler: current_cpu_utilization_percentage: [ {} ] is invalid.'.format(pct))
        self._current_cpu_utilization_percentage = pct
            
    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.observed_generation is not None:
            data['observedGeneration'] = self.observed_generation
        if self.last_scale_time is not None:
            data['lastScaleTime'] = self.last_scale_time
        if self.current_replicas is not None:
            data['currentReplicas'] = self.current_replicas
        if self.desired_replicas is not None:
            data['desiredReplicas'] = self.desired_replicas
        if self.current_cpu_utilization_percentage is not None:
            data['currentCpuUtilizationPercentage'] = self.current_cpu_utilization_percentage
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, is_valid_list
from kubernetes.models.v1.KeyToPath import KeyToPath
from kubernetes.utils import filter_model


class SecretVolumeSource(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_secretvolumesource
    """

    def __init__(self, model=None):
        super(SecretVolumeSource, self).__init__()

        self._secret_name = None
        self._items = None
        self._default_mode = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'secretName' in model:
            self.secret_name = model['secretName']
        if 'items' in model:
            items = []
            for i in model['items']:
                item = KeyToPath(i)
                items.append(item)
            self.items = items
        if 'defaultMode' in model:
            self.default_mode = model['defaultMode']

    # ------------------------------------------------------------------------------------- secret name

    @property
    def secret_name(self):
        return self._secret_name

    @secret_name.setter
    def secret_name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('SecretVolumeSource: secret_name: [ {0} ] is invalid.'.format(name))
        self._secret_name = name

    # ------------------------------------------------------------------------------------- items

    @property
    def items(self):
        return self._items

    @items.setter
    def items(self, items=None):
        if not is_valid_list(items, KeyToPath):
            raise SyntaxError('SecretVolumeSource: items: [ {0} ] is invalid.'.format(items))
        self._items = items

    # ------------------------------------------------------------------------------------- defaultMode

    @property
    def default_mode(self):
        return self._default_mode

    @default_mode.setter
    def default_mode(self, mode=None):
        if not isinstance(mode, int):
            raise SyntaxError('SecretVolumeSource: default_mode: [ {0} ] is invalid.'.format(mode))
        self._default_mode = mode

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.secret_name is not None:
            data['secretName'] = self.secret_name
        if self.items is not None:
            data['items'] = [x.serialize() for x in self.items]
        if self.default_mode is not None:
            data['defaultMode'] = self.default_mode
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, filter_model


class ContainerStateTerminated(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_containerstateterminated
    """

    def __init__(self, model=None):
        super(ContainerStateTerminated, self).__init__()

        self._exit_code = None
        self._signal = None
        self._reason = None
        self._message = None
        self._started_at = None
        self._finished_at = None
        self._container_id = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'exitCode' in model:
            self.exit_code = model['exitCode']
        if 'signal' in model:
            self.signal = model['signal']
        if 'reason' in model:
            self.reason = model['reason']
        if 'message' in model:
            self.message = model['message']
        if 'startedAt' in model:
            self.started_at = model['startedAt']
        if 'finishedAt' in model:
            self.finished_at = model['finishedAt']
        if 'containerID' in model:
            self.container_id = model['containerID']

    # ------------------------------------------------------------------------------------- exit code

    @property
    def exit_code(self):
        return self._exit_code

    @exit_code.setter
    def exit_code(self, code=None):
        if not isinstance(code, int):
            raise SyntaxError('ContainerStateTerminated: exit_code: [ {0} ] is invalid.'.format(code))
        self._exit_code = code

    # ------------------------------------------------------------------------------------- signal

    @property
    def signal(self):
        return self._signal

    @signal.setter
    def signal(self, signal=None):
        if not isinstance(signal, int):
            raise SyntaxError('ContainerStateTerminated: signal: [ {0} ] is invalid.'.format(signal))
        self._signal = signal

    # ------------------------------------------------------------------------------------- reason

    @property
    def reason(self):
        return self._reason

    @reason.setter
    def reason(self, msg=None):
        if not is_valid_string(msg):
            raise SyntaxError('ContainerStateTerminated: reason: [ {0} ] is invalid.'.format(msg))
        self._reason = msg

    # ------------------------------------------------------------------------------------- message

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, msg=None):
        if not is_valid_string(msg):
            raise SyntaxError('ContainerStateTerminated: message: [ {0} ] is invalid.'.format(msg))
        self._message = msg

    # ------------------------------------------------------------------------------------- started at

    @property
    def started_at(self):
        return self._started_at

    @started_at.setter
    def started_at(self, time=None):
        if not is_valid_string(time):
            raise SyntaxError('ContainerStateTerminated: started_at: [ {0} ] is invalid.'.format(time))
        self._started_at = time

    # ------------------------------------------------------------------------------------- finished at

    @property
    def finished_at(self):
        return self._finished_at

    @finished_at.setter
    def finished_at(self, time=None):
        if not is_valid_string(time):
            raise SyntaxError('ContainerStateTerminated: finished_at: [ {0} ] is invalid.'.format(time))
        self._finished_at = time

    # ------------------------------------------------------------------------------------- containerID

    @property
    def container_id(self):
        return self._container_id

    @container_id.setter
    def container_id(self, cid=None):
        if not is_valid_string(cid):
            raise SyntaxError('ContainerStateTerminated: container_id: [ {0} ] is invalid.'.format(cid))
        self._container_id = cid

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.exit_code is not None:
            data['exitCode'] = self.exit_code
        if self.signal is not None:
            data['signal'] = self.signal
        if self.reason is not None:
            data['reason'] = self.reason
        if self.message is not None:
            data['message'] = self.message
        if self.started_at is not None:
            data['startedAt'] = self.started_at
        if self.finished_at is not None:
            data['finishedAt'] = self.finished_at
        if self.container_id is not None:
            data['containerID'] = self.container_id
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.ObjectReference import ObjectReference
from kubernetes.utils import is_valid_list, is_valid_string


class CronJobStatus(object):

    def __init__(self, model=None):
        super(CronJobStatus, self).__init__()

        self._active = []
        self._successful = None
        self._failed = None
        self._last_schedule_time = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'active' in model:
            refs = []
            for j in model['active']:
                ref = ObjectReference(j)
                refs.append(ref)
            self.active = refs
        if 'successful' in model:
            self.successful = model['successful']
        if 'failed' in model:
            self.failed = model['failed']
        if 'lastScheduleTime' in model:
            self.last_schedule_time = model['lastScheduleTime']

    # -------------------------------------------------------------------------------------  active

    @property
    def active(self):
        return self._active

    @active.setter
    def active(self, _list=None):
        if not is_valid_list(_list, ObjectReference):
            raise SyntaxError('CronJobStatus: active: [ {} ] is invalid.'.format(_list))
        self._active = _list

    # -------------------------------------------------------------------------------------  successful

    @property
    def successful(self):
        return self._successful

    @successful.setter
    def successful(self, s=None):
        if not isinstance(s, bool):
            raise SyntaxError('CronJobStatus: successful: [ {} ] is invalid.'.format(s))
        self._successful = s

    # -------------------------------------------------------------------------------------  failed

    @property
    def failed(self):
        return self._failed

    @failed.setter
    def failed(self, f=None):
        if not isinstance(f, bool):
            raise SyntaxError('CronJobStatus: failed: [ {} ] is invalid.'.format(f))
        self._failed = f

    # -------------------------------------------------------------------------------------  lastScheduleTime

    @property
    def last_schedule_time(self):
        return self._last_schedule_time

    @last_schedule_time.setter
    def last_schedule_time(self, t=None):
        if not is_valid_string(t):
            raise SyntaxError('CronJobStatus: last_schedule_time: [ {} ] is invalid.'.format(t))
        self._last_schedule_time = t

    # -------------------------------------------------------------------------------------  serialize

    def serialize(self):
        data = {}
        if self.active is not None:
            data['active'] = [x.serialize() for x in self.active]
        if self.successful is not None:
            data['successful'] = self.successful
        if self.failed is not None:
            data['failed'] = self.failed
        if self.last_schedule_time is not None:
            data['lastScheduleTime'] = self.last_schedule_time
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import json

from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1.HorizontalPodAutoscaler import HorizontalPodAutoscaler
from kubernetes.models.v1beta1.SubresourceReference import SubresourceReference
from kubernetes.utils import convert


class K8sHorizontalPodAutoscaler(K8sObject):

    VALID_HPA_TARGET_KINDS = ['ReplicationController', 'Deployment']

    def __init__(self, config=None, name=None):

        super(K8sHorizontalPodAutoscaler, self).__init__(
            config=config,
            obj_type='HorizontalPodAutoscaler',
            name=name
        )

    # -------------------------------------------------------------------------------------  override

    def create(self):
        super(K8sHorizontalPodAutoscaler, self).create()
        self.get()
        return self

    def update(self):
        super(K8sHorizontalPodAutoscaler, self).update()
        self.get()
        return self

    def list(self, pattern=None):
        ls = super(K8sHorizontalPodAutoscaler, self).list()
        hpas = list(map(lambda x: HorizontalPodAutoscaler(x), ls))
        if pattern is not None:
            hpas = list(filter(lambda x: pattern in x.name, hpas))
        k8s = []
        for x in hpas:
            z = K8sHorizontalPodAutoscaler(config=self.config, name=x.name)
            z.model = x
            k8s.append(z)
        return k8s

    # ------------------------------------------------------------------------------------- get

    def get(self):
        self.model = HorizontalPodAutoscaler(self.get_model())
        return self

    # ------------------------------------------------------------------------------------- cpu_percent

    @property
    def cpu_percent(self):
        return self.model.spec.cpu_utilization

    @cpu_percent.setter
    def cpu_percent(self, pct=None):
        self.model.spec.cpu_utilization = pct

    # ------------------------------------------------------------------------------------- min replicas

    @property
    def min_replicas(self):
        return self.model.spec.min_replicas

    @min_replicas.setter
    def min_replicas(self, min=None):
        self.model.spec.min_replicas = min

    # ------------------------------------------------------------------------------------- max replicas

    @property
    def max_replicas(self):
        return self.model.spec.max_replicas

    @max_replicas.setter
    def max_replicas(self, max=None):
        self.model.spec.max_replicas = max

    # ------------------------------------------------------------------------------------- scaleRef

    @property
    def scale_ref(self):
        return self.model.spec.scale_target_ref

    @scale_ref.setter
    def scale_ref(self, ref=None):
        if not isinstance(ref, tuple):
            raise SyntaxError(
                'K8sHorizontalPodAutoscaler: scale_ref must be a tuple of the form (kind, name).')

        kind, name = ref
        if kind not in self.VALID_HPA_TARGET_KINDS:
            raise SyntaxError(
                'K8sHorizontalPodAutoscaler: scale_ref.kind: [ {} ] is invalid.'.format(kind))

        ref = {
            'apiVersion': 'v1' if kind == 'ReplicationController' else 'extensions/v1beta1',
            'kind': kind,
            'name': name
        }

        sub = SubresourceReference(ref)
        self.model.spec.scale_target_ref = sub

    # ------------------------------------------------------------------------------------- from_json

    @staticmethod
    def from_json(j=None, cfg=None):
        try:
            d = convert(json.loads(j))
            model = HorizontalPodAutoscaler(model=d)
            k8s = K8sHorizontalPodAutoscaler(config=cfg, name="yo")
            k8s.model = model
            return k8s

        except TypeError as err:
            raise SyntaxError(
                'K8sHorizontalPodAutoscaler: json: [ {} ] is invalid: [ {} ]'.format(j, err))

        except ValueError as err:
            raise SyntaxError(
                'K8sHorizontalPodAutoscaler: json: [ {} ] is invalid: [ {} ]'.format(j, err))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#


class CPUTargetUtilization(object):
    """
    https://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_cputargetutilization
    """

    def __init__(self, model=None):
        super(CPUTargetUtilization, self).__init__()

        self._target_percentage = 70

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'targetPercentage' in model:
            self.target_percentage = model['targetPercentage']

    # ------------------------------------------------------------------------------------- targetPercentage

    @property
    def target_percentage(self):
        return self._target_percentage
    
    @target_percentage.setter
    def target_percentage(self, t=None):
        if not isinstance(t, int):
            raise SyntaxError("CPUTargetUtilization: target_percentage: [ {} ] is invalid.".format(t))
        self._target_percentage = t



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class ObjectFieldSelector(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_objectfieldselector
    """

    def __init__(self, model=None):
        super(ObjectFieldSelector, self).__init__()

        self._api_version = None
        self._field_path = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'apiVersion' in model:
            self.api_version = model['apiVersion']
        if 'fieldPath' in model:
            self.field_path = model['fieldPath']

    # ------------------------------------------------------------------------------------- apiVersion

    @property
    def api_version(self):
        return self._api_version

    @api_version.setter
    def api_version(self, v=None):
        if not is_valid_string(v):
            raise SyntaxError('ObjectFieldSelector: api_version: [ {} ] is invalid.'.format(v))
        self._api_version = v

    # ------------------------------------------------------------------------------------- fieldPath

    @property
    def field_path(self):
        return self._field_path

    @field_path.setter
    def field_path(self, fp=None):
        if not is_valid_string(fp):
            raise SyntaxError('ObjectFieldSelector: field_path: [ {} ] is invalid.'.format(fp))
        self._field_path = fp

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.api_version is not None:
            data['apiVersion'] = self.api_version
        if self.field_path is not None:
            data['fieldPath'] = self.field_path
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class ServicePort(object):

    VALID_PROTOCOLS = ['TCP', 'UDP']

    def __init__(self, name=None, model=None):
        super(ServicePort, self).__init__()

        self._name = None
        self._protocol = None
        self._port = None
        self._target_port = None
        self._node_port = None

        if name is not None:
            self.name = name

        if model is not None:
            if 'name' in model:
                self.name = model['name']
            if 'protocol' in model:
                self.protocol = model['protocol']
            if 'port' in model:
                self.port = model['port']
            if 'targetPort' in model:
                self.target_port = model['targetPort']
            if 'nodePort' in model:
                self.node_port = model['nodePort']

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('ServicePort: name: [ {} ] is invalid.'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- protocol

    @property
    def protocol(self):
        return self._protocol

    @protocol.setter
    def protocol(self, protocol=None):
        if not is_valid_string(protocol) or protocol.upper() not in ServicePort.VALID_PROTOCOLS:
            raise SyntaxError('ServicePort: protocol: [ {} ] is invalid.'.format(protocol))
        self._protocol = protocol.upper()

    # ------------------------------------------------------------------------------------- port

    @property
    def port(self):
        return self._port

    @port.setter
    def port(self, port=None):
        if isinstance(port, str) and port.isdigit():
            port = int(port)
        if not isinstance(port, int):
            raise SyntaxError('ServicePort: port: [ {} ] is invalid.'.format(port))
        self._port = port

    # ------------------------------------------------------------------------------------- targetPort

    @property
    def target_port(self):
        return self._target_port

    @target_port.setter
    def target_port(self, port=None):
        msg = 'ServicePort: target_port: [ {} ] is invalid.'.format(port)
        try:
            p = int(port)
        except ValueError:
            if not is_valid_string(port):
                raise SyntaxError(msg)
            p = port
        except TypeError:
            raise SyntaxError(msg)
        self._target_port = p

    # ------------------------------------------------------------------------------------- nodePort

    @property
    def node_port(self):
        return self._node_port

    @node_port.setter
    def node_port(self, port=None):
        if port is not None and isinstance(port, str) and port.isdigit():
            port = int(port)
        if port is not None and not isinstance(port, int):
            raise SyntaxError('ServicePort: node_port: [ {} ] is invalid.'.format(port))
        self._node_port = port

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.name is not None:
            data['name'] = self.name
        if self.protocol is not None:
            data['protocol'] = self.protocol
        if self.port is not None:
            data['port'] = self.port
        if self.target_port is not None:
            data['targetPort'] = self.target_port
        if self.node_port is not None:
            data['nodePort'] = self.node_port
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import time

from kubernetes.K8sContainer import K8sContainer
from kubernetes.K8sExceptions import CronJobAlreadyRunningException, CronJobRunException, TimedOutException
from kubernetes.K8sObject import K8sObject
from kubernetes.K8sPod import K8sPod
from kubernetes.K8sVolume import K8sVolume
from kubernetes.models.v2alpha1.CronJob import CronJob
from kubernetes.utils import is_valid_list, is_reachable, sanitize_version_number


class K8sCronJob(K8sObject):

    POD_RUN_WAIT_TIMEOUT_SECONDS = 600

    def __init__(self, config=None, name=None):

        temp = K8sObject(config=config, obj_type='Pod', name='temp')
        _type = 'CronJob'

        if config and is_reachable(config):
            v = temp.server_version()
            if sanitize_version_number(v['major']) == 1 and sanitize_version_number(v['minor']) == 4:
                _type = 'ScheduledJob'

        super(K8sCronJob, self).__init__(
            config=config,
            obj_type=_type,
            name=name
        )

        if config is not None and config.pull_secret is not None:
            self.add_image_pull_secrets(config.pull_secret)

    # -------------------------------------------------------------------------------------  override

    def create(self):
        super(K8sCronJob, self).create()
        self.get()
        return self

    def update(self):
        super(K8sCronJob, self).update()
        self.get()
        return self

    def list(self, pattern=None):
        ls = super(K8sCronJob, self).list()
        jobs = list(map(lambda x: CronJob(x), ls))
        if pattern is not None:
            jobs = list(filter(lambda x: pattern in x.name, jobs))
        k8s = []
        for x in jobs:
            j = K8sCronJob(config=self.config, name=x.name)
            j.model = x
            k8s.append(j)
        return k8s

    # -------------------------------------------------------------------------------------  get

    def get(self):
        self.model = CronJob(self.get_model())
        return self

    # -------------------------------------------------------------------------------------  add

    def add_container(self, container=None):
        if not isinstance(container, K8sContainer):
            raise SyntaxError(
                'K8sCronJob.add_container() container: [ {0} ] is invalid.'.format(container))

        containers = self.model.spec.job_template.spec.template.spec.containers
        if containers is None:
            containers = []
        filtered = list(filter(lambda x: x.name != container.name, containers))
        filtered.append(container.model)
        self.model.spec.job_template.spec.template.spec.containers = filtered
        return self

    def add_image_pull_secrets(self, secrets=None):
        self.model.spec.job_template.spec.template.spec.add_image_pull_secrets(secrets)
        return self

    def add_volume(self, volume=None):
        if not isinstance(volume, K8sVolume):
            raise SyntaxError(
                'K8sCronJob.add_volume() volume: [ {0} ] is invalid.'.format(volume))

        volumes = self.model.spec.job_template.spec.template.spec.volumes
        if volume.model not in volumes:
            volumes.append(volume.model)
        self.model.spec.job_template.spec.template.spec.volumes = volumes
        return self

    # -------------------------------------------------------------------------------------  schedule

    @property
    def schedule(self):
        return self.model.spec.schedule

    @schedule.setter
    def schedule(self, s=None):
        self.model.spec.schedule = s

    # -------------------------------------------------------------------------------------  startingDeadlineSeconds

    @property
    def starting_deadline_seconds(self):
        return self.model.spec.starting_deadline_seconds

    @starting_deadline_seconds.setter
    def starting_deadline_seconds(self, s=None):
        self.model.spec.starting_deadline_seconds = s

    # -------------------------------------------------------------------------------------  activeDeadlineSeconds

    @property
    def active_deadline_seconds(self):
        return self.model.spec.job_template.spec.active_deadline_seconds

    @active_deadline_seconds.setter
    def active_deadline_seconds(self, s=None):
        self.model.spec.job_template.spec.active_deadline_seconds = s

    # -------------------------------------------------------------------------------------  concurrencyPolicy

    @property
    def concurrency_policy(self):
        return self.model.spec.concurrency_policy

    @concurrency_policy.setter
    def concurrency_policy(self, cp=None):
        self.model.spec.concurrency_policy = cp

    # -------------------------------------------------------------------------------------  suspend

    @property
    def suspend(self):
        return self.model.spec.suspend

    @suspend.setter
    def suspend(self, s=None):
        self.model.spec.suspend = s

    # -------------------------------------------------------------------------------------  parallelism

    @property
    def parallelism(self):
        return self.model.spec.job_template.spec.parallelism

    @parallelism.setter
    def parallelism(self, jp=None):
        self.model.spec.job_template.spec.parallelism = jp

    # -------------------------------------------------------------------------------------  completions

    @property
    def completions(self):
        return self.model.spec.job_template.spec.completions

    @completions.setter
    def completions(self, c=None):
        self.model.spec.job_template.spec.completions = c

    # -------------------------------------------------------------------------------------  containers

    @property
    def containers(self):
        objs = []
        for c in self.model.spec.job_template.spec.template.spec.containers:
            k8scontainer = K8sContainer(name=c.name, image=c.image)
            k8scontainer.model = c
            objs.append(k8scontainer)
        return objs

    @containers.setter
    def containers(self, containers=None):
        if not is_valid_list(containers, K8sContainer):
            raise SyntaxError(
                'K8sCronJob: containers: [ {} ] is invalid.'.format(containers))

        models = []
        for obj in containers:
            models.append(obj.model)
        self.model.spec.job_template.spec.template.spec.containers = models

    # -------------------------------------------------------------------------------------  container_image

    @property
    def container_image(self):
        data = {}
        for c in self.containers:
            data[c.name] = c.image
        return data

    @container_image.setter
    def container_image(self, tup=None):
        if not isinstance(tup, tuple):
            raise SyntaxError(
                'K8sCronJob.container_image() must be a tuple of the form (name, image)')

        name, image = tup
        found = list(filter(lambda x: x.name == name, self.containers))
        if found:
            new = list(filter(lambda x: x.name != name, self.containers))
            found[0].image = image
            new.append(found[0])
            self.containers = new

    # -------------------------------------------------------------------------------------  dnsPolicy

    @property
    def dns_policy(self):
        return self.model.spec.template.spec.dns_policy

    @dns_policy.setter
    def dns_policy(self, policy=None):
        if policy not in self.model.spec.job_template.spec.template.spec.VALID_DNS_POLICIES:
            raise SyntaxError(
                'K8sJob: dns_policy: [ {} ] is invalid, expected [ {} ].'.format(
                    policy, self.model.spec.job_template.spec.template.spec.VALID_DNS_POLICIES))

        self.model.spec.job_template.spec.template.spec.dns_policy = policy

    # -------------------------------------------------------------------------------------  restartPolicy

    @property
    def restart_policy(self):
        return self.model.spec.job_template.spec.template.spec.restart_policy

    @restart_policy.setter
    def restart_policy(self, p=None):
        self.model.spec.job_template.spec.template.spec.restart_policy = p

    # -------------------------------------------------------------------------------------  lastScheduleTime

    @property
    def last_schedule_time(self):
        return self.model.status.last_schedule_time

    @last_schedule_time.setter
    def last_schedule_time(self, t=None):
        raise NotImplementedError(
            'K8sCronJob: last_schedule_time is read-only.')

    # -------------------------------------------------------------------------------------  active

    @property
    def active(self):
        return self.status.active

    @active.setter
    def active(self, a=None):
        raise NotImplementedError(
            "K8sCronJob: active is read-only.")

    # -------------------------------------------------------------------------------------  pod

    @property
    def pod(self):
        podspec = self.model.spec.job_template.spec.template.spec
        name = "{}-{}".format(self.name, int(time.time()))
        pod = K8sPod(config=self.config, name=name)
        pod.model.spec = podspec
        return pod

    @pod.setter
    def pod(self, p=None):
        raise NotImplementedError(
            'K8sCronjob: pod is read-only.')

    # -------------------------------------------------------------------------------------  image pull secrets

    @property
    def image_pull_secrets(self):
        return self.model.spec.job_template.spec.template.spec.image_pull_secrets

    @image_pull_secrets.setter
    def image_pull_secrets(self, secrets=None):
        self.model.spec.job_template.spec.template.spec.image_pull_secrets = secrets

    # -------------------------------------------------------------------------------------  run

    def run(self, timeout=POD_RUN_WAIT_TIMEOUT_SECONDS):
        """
        Forces a K8sCronJob to run immediately.

        - Fail if the K8sCronJob is currently running on-schedule.
        - Suspend the K8sCronJob.
        - Spawn a K8sPod.
        - Unsuspend a K8sCronJob.

        :param timeout: The timeout, in seconds, after which to kill the K8sPod.

        :return: None.
        """

        if not isinstance(timeout, int):
            raise SyntaxError("K8sCronJob.run() timeout: [ {} ] is invalid.")

        if len(self.active):
            raise CronJobAlreadyRunningException(
                "K8sCronJob.run() failed: CronJob: [ {} ] "
                "has [ {} ] active Jobs currently.".format(self.name, len(self.active)))

        self.suspend = True
        self.update()

        pod = self.pod
        if timeout:
            self.POD_RUN_WAIT_TIMEOUT_SECONDS = timeout

        try:
            pod.create()
            start_time = time.time()
            while pod.phase not in ['Succeeded', 'Failed']:
                pod.get()
                time.sleep(2)
                self._check_timeout(start_time)

        except Exception as err:
            raise CronJobRunException("K8sCronJob.run() failed: {}".format(err))

        finally:
            pod.delete()
            self.suspend = False
            self.update()

    def _check_timeout(self, start_time=None):
        if self.POD_RUN_WAIT_TIMEOUT_SECONDS == -1:
            return
        elapsed_time = time.time() - start_time
        if elapsed_time >= self.POD_RUN_WAIT_TIMEOUT_SECONDS:  # timeout
            raise TimedOutException(
                "Timed out running one-off CronJob: [ {} ]".format(self.name))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import re

from kubernetes.models.unversioned.BaseModel import BaseModel


class ConfigMap(BaseModel):
    """
    https://kubernetes.io/docs/api-reference/v1.8/#configmap-v1-core
    """

    def __init__(self, model=None):
        super(ConfigMap, self).__init__()

        self.kind = 'ConfigMap'
        self.api_version = 'v1'

        self._data = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        super(ConfigMap, self).build_with_model(model)

        if 'data' in model:
            self.data = model['data']

    # --------------------------------------------------------------------------------- data

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, v=None):
        if not isinstance(v, dict):
            raise SyntaxError('ConfigMap: data: [ {} ] is invalid.'.format(v))
        pattern = re.compile(r'^[0-9a-zA-Z\-\.\_]+$')
        for k in v.keys():
            if not pattern.match(k):
                raise SyntaxError('ConfigMap: data: Key [ {} ] is invalid.'.format(k))
        self._data = v

    def serialize(self):
        tmp_data = super(ConfigMap, self).serialize()

        if self.data is not None:
            tmp_data['data'] = self.data
        return tmp_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, filter_model


class NFSVolumeSource(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_nfsvolumesource
    """

    def __init__(self, model=None):
        super(NFSVolumeSource, self).__init__()

        self._server = None
        self._path = None
        self._read_only = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model):
        if 'server' in model:
            self.server = model['server']
        if 'path' in model:
            self.path = model['path']
        if 'readOnly' in model:
            self.read_only = model['readOnly']

    # ------------------------------------------------------------------------------------- server

    @property
    def server(self):
        return self._server

    @server.setter
    def server(self, server=None):
        if not is_valid_string(server):
            raise SyntaxError('NFSVolumeSource: server: [ {0} ] is invalid.'.format(server))
        self._server = server

    # ------------------------------------------------------------------------------------- path

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, path=None):
        if not is_valid_string(path):
            raise SyntaxError('NFSVolumeSource: path: [ {0} ] is invalid.'.format(path))
        self._path = path

    # ------------------------------------------------------------------------------------- readOnly

    @property
    def read_only(self):
        return self._read_only

    @read_only.setter
    def read_only(self, ro=None):
        if not isinstance(ro, bool):
            raise SyntaxError('NFSVolumeSource: read_only: [ {0} ] is invalid.'.format(ro))
        self._read_only = ro

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.server is not None:
            data['server'] = self.server
        if self.path is not None:
            data['path'] = self.path
        if self.read_only is not None:
            data['readOnly'] = self.read_only
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1beta1.SubresourceReference import SubresourceReference


class HorizontalPodAutoscalerSpec(object):
    """
    https://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_horizontalpodautoscalerspec
    """

    def __init__(self, model=None):
        super(HorizontalPodAutoscalerSpec, self).__init__()

        self._scale_target_ref = SubresourceReference()
        self._min_replicas = 1
        self._max_replicas = 1  # cannot be smaller than min_replicas
        self._cpu_utilization = 70

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'scaleTargetRef' in model:
            self.scale_target_ref = SubresourceReference(model['scaleTargetRef'])
        if 'minReplicas' in model:
            self.min_replicas = model['minReplicas']
        if 'maxReplicas' in model:
            self.max_replicas = model['maxReplicas']
        if 'targetCPUUtilizationPercentage' in model:
            self.cpu_utilization = model['targetCPUUtilizationPercentage']

    # ------------------------------------------------------------------------------------- scaleRef

    @property
    def scale_target_ref(self):
        return self._scale_target_ref

    @scale_target_ref.setter
    def scale_target_ref(self, ref=None):
        if not isinstance(ref, SubresourceReference):
            raise SyntaxError('HorizontalPodAutoscaler: scale_target_ref: [ {} ] is invalid.'.format(ref))
        self._scale_target_ref = ref

    # ------------------------------------------------------------------------------------- minReplicas
        
    @property
    def min_replicas(self):
        return self._min_replicas

    @min_replicas.setter
    def min_replicas(self, r=None):
        if not isinstance(r, int):
            raise SyntaxError('HorizontalPodAutoscaler: min_replicas: [ {} ] is invalid.'.format(r))
        self._min_replicas = r

    # ------------------------------------------------------------------------------------- maxReplicas

    @property
    def max_replicas(self):
        return self._max_replicas

    @max_replicas.setter
    def max_replicas(self, r=None):
        if not isinstance(r, int):
            raise SyntaxError('HorizontalPodAutoscaler: max_replicas: [ {} ] is invalid.'.format(r))
        self._max_replicas = r

    # ------------------------------------------------------------------------------------- cpuUtilization

    @property
    def cpu_utilization(self):
        return self._cpu_utilization

    @cpu_utilization.setter
    def cpu_utilization(self, u=None):
        if not isinstance(u, int):
            raise SyntaxError('HorizontalPodAutoscaler: cpu_utilization: [ {} ] is invalid.'.format(u))
        self._cpu_utilization = u

    # ------------------------------------------------------------------------------------- serialize()

    def serialize(self):
        data = {}
        if self.scale_target_ref is not None:
            data['scaleTargetRef'] = self.scale_target_ref.serialize()
        if self.min_replicas is not None:
            data['minReplicas'] = self.min_replicas
        if self.max_replicas is not None:
            data['maxReplicas'] = self.max_replicas
        if self.cpu_utilization is not None:
            data['targetCPUUtilizationPercentage'] = self.cpu_utilization
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import collections
from six import string_types


def convert(data):
    if isinstance(data, string_types):
        return str(data)
    elif isinstance(data, collections.Mapping):
        return dict(map(convert, data.items()))
    elif isinstance(data, collections.Iterable):
        return type(data)(map(convert, data))
    else:
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import json
import time
from dateutil import parser

import yaml

from kubernetes.K8sConfig import K8sConfig
from kubernetes.K8sExceptions import *
from kubernetes.models.unversioned.BaseUrls import BaseUrls
from kubernetes.models.v1.DeleteOptions import DeleteOptions
from kubernetes.utils import HttpRequest, is_valid_dict, str_to_class

VALID_K8s_OBJS = [
    'ComponentStatus',
    'ConfigMap',
    'CronJob',  # server version >= 1.5
    'DaemonSet',
    'Deployment',
    'Event',
    'HorizontalPodAutoscaler',
    'Job',
    'Namespace',
    'Node',
    'PersistentVolume',
    'PersistentVolumeClaim',
    'PetSet',  # server version == 1.4
    'Pod',
    'ReplicaSet',
    'ReplicationController',
    'ScheduledJob',  # server version == 1.4
    'Secret',
    'Service',
    'ServiceAccount',
    'StatefulSet',  # server version >= 1.5
    'StorageClass',
    'Volume'
]


class K8sObject(object):

    DELETE_TIMEOUT_SECONDS = 60

    def __init__(self, config=None, obj_type=None, name=None):
        super(K8sObject, self).__init__()

        if config is not None and not isinstance(config, K8sConfig):
            raise SyntaxError('K8sObject: config: [ {0} ] must be of type K8sConfig.'.format(config.__class__.__name__))
        if config is None:
            config = K8sConfig()
        self.config = config

        if obj_type not in VALID_K8s_OBJS:
            valid = ", ".join(VALID_K8s_OBJS)
            raise InvalidObjectException('K8sObject: obj_type: [ {0} ] must be in: [ {1} ]'.format(obj_type, valid))
        self.obj_type = obj_type

        self.model = str_to_class(obj_type)
        self.name = name

        try:
            urls = BaseUrls(api=self.config.version, namespace=self.config.namespace)
            self.base_url = urls.get_base_url(object_type=obj_type)
        except Exception as err:
            raise Exception('Could not set BaseUrl for type: [ {0} ]'.format(obj_type))

    def __str__(self):
        return "{}".format(self.model.serialize())

    def __eq__(self, other):
        # see https://github.com/kubernetes/kubernetes/blob/release-1.3/docs/design/identifiers.md
        if isinstance(other, self.__class__):
            return self.config.namespace == other.config.namespace \
                   and self.name == other.name \
                   and self.uid == other.uid
        return NotImplemented

    # ------------------------------------------------------------------------------------- get

    def get_annotation(self, k=None):
        if k in self.model.metadata.annotations:
            return self.model.metadata.annotations[k]
        return None

    def get_label(self, k=None):
        if k in self.model.metadata.labels:
            return self.model.metadata.labels[k]
        return None

    # ------------------------------------------------------------------------------------- add

    def add_annotation(self, k=None, v=None):
        anns = self.model.metadata.annotations
        if anns is None:
            anns = {}
        anns.update({k: str(v)})
        self.model.metadata.annotations = anns
        return self

    def add_label(self, k=None, v=None):
        labels = self.model.metadata.labels
        if labels is None:
            labels = {}
        labels.update({k: v})
        self.model.metadata.labels = labels
        return self

    # ------------------------------------------------------------------------------------- del

    def del_annotation(self, k=None):
        orig = self.model.metadata.annotations
        if k in orig:
            orig.pop(k)
            self.model.metadata.annotations = orig
        return self

    def del_label(self, k=None):
        orig = self.model.metadata.labels
        if k in orig:
            orig.pop(k)
            self.model.metadata.labels = orig
        return self

    # ------------------------------------------------------------------------------------- annotations

    @property
    def annotations(self):
        return self.model.metadata.annotations

    @annotations.setter
    def annotations(self, anns=None):
        self.model.metadata.annotations = anns

    # -------------------------------------------------------------------------------------  creationTimestamp

    @property
    def creation_timestamp(self):
        datestring = self.model.metadata.creation_timestamp
        dt = parser.parse(datestring)
        return dt

    @creation_timestamp.setter
    def creation_timestamp(self, t=None):
        raise NotImplementedError('K8sObject: creation_timestamp is read-only.')

    # -------------------------------------------------------------------------------------  createdBy

    @property
    def created_by(self):
        if 'kubernetes.io/created-by' in self.annotations:
            ref = json.loads(self.annotations['kubernetes.io/created-by'])
            obj = K8sObject(
                config=self.config,
                name=ref['reference']['name'],
                obj_type=ref['reference']['kind'])
            obj.get_model()
            return obj
        return None

    @created_by.setter
    def created_by(self, cb=None):
        raise NotImplementedError('K8sObject: created_by is read-only.')

    # ------------------------------------------------------------------------------------- labels

    @property
    def labels(self):
        return self.model.metadata.labels

    @labels.setter
    def labels(self, labels=None):
        self.model.metadata.labels = labels

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self.model.metadata.name

    @name.setter
    def name(self, name=None):
        self.model.metadata.name = name
        self.model.metadata.labels['name'] = name

    # ------------------------------------------------------------------------------------- status

    @property
    def status(self):
        return self.model.status

    @status.setter
    def status(self, status=None):
        raise NotImplementedError()

    # ------------------------------------------------------------------------------------- uid

    @property
    def uid(self):
        return self.model.metadata.uid

    @uid.setter
    def uid(self, uid=None):
        raise NotImplementedError("K8sObject: uid is read-only.")

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        return self.model.serialize()

    def as_json(self):
        data = self.serialize()
        dump = json.dumps(data, indent=4)
        return dump

    def as_yaml(self):
        data = self.serialize()
        dump = yaml.dump(data, default_flow_style=False)
        return dump

    # ------------------------------------------------------------------------------------- remote API calls

    def request(self, method='GET', host=None, url=None, auth=None, cert=None,
                data=None, token=None, ca_cert=None, ca_cert_data=None):

        host = self.config.api_host if host is None else host
        url = self.base_url if url is None else url
        auth = self.config.auth if auth is None else auth
        cert = self.config.cert if cert is None else cert
        token = self.config.token if token is None else token
        ca_cert = self.config.ca_cert if ca_cert is None else ca_cert
        ca_cert_data = self.config.ca_cert_data if ca_cert_data is None else ca_cert_data

        r = HttpRequest(
            method=method,
            host=host,
            url=url,
            auth=auth,
            cert=cert,
            ca_cert=ca_cert,
            ca_cert_data=ca_cert_data,
            data=data,
            token=token
        )

        try:
            return r.send()
        except IOError as err:
            raise BadRequestException('K8sObject: IOError: {0}'.format(err))

    def list(self):
        state = self.request(method='GET')
        if not state.get('status'):
            raise Exception('K8sObject: Could not fetch list of objects of type: [ {0} ]'.format(self.obj_type))
        if not state.get('success'):
            status = state.get('status', '')
            state_data = state.get('data', dict())
            reason = state_data['message'] if 'message' in state_data else state_data
            message = 'K8sObject: LIST failed : HTTP {0} : {1}'.format(status, reason)
            if int(status) == 401:
                raise UnauthorizedException(message)
            if int(status) == 409:
                raise AlreadyExistsException(message)
            if int(status) == 422:
                raise UnprocessableEntityException(message)
            raise BadRequestException(message)
        items = state.get('data', dict()).get('items', list())
        return items if items is not None else []

    def get_model(self):
        if self.name is None:
            raise SyntaxError('K8sObject: name: [ {0} ] must be set to fetch the object.'.format(self.name))

        url = '{base}/{name}'.format(base=self.base_url, name=self.name)
        state = self.request(method='GET', url=url)

        if not state.get('success'):
            status = state.get('status', '')
            reason = state.get('data', dict()).get('message', None)
            message = 'K8sObject: GET [ {0}:{1} ] failed: HTTP {2} : {3} '.format(
                self.obj_type, self.name, status, reason)
            raise NotFoundException(message)

        model = state.get('data')
        return model

    def get_with_params(self, data=None):
        if not is_valid_dict(data):
            raise SyntaxError('K8sObject.get_with_params(): data: [ {0} ] is invalid.'.format(data))
        url = '{base}'.format(base=self.base_url)
        state = self.request(method='GET', url=url, data=data)
        items = state.get('data', None).get('items', list())
        if items is None:
            return []
        return items

    def get_exportable(self):
        if self.name is None:
            raise SyntaxError('K8sObject: name: [ {0} ] must be set to fetch the object.'.format(self.name))

        url = '{base}/{name}?export=true'.format(base=self.base_url, name=self.name)
        state = self.request(method='GET', url=url)

        if not state.get('success'):
            status = state.get('status', '')
            reason = state.get('data', dict()).get('message', None)
            message = 'K8sObject: GET [ {0}:{1} ] failed: HTTP {2} : {3} '.format(
                self.obj_type, self.name, status, reason)
            raise NotFoundException(message)

        data = state.get('data')
        return data

    def create(self):
        if self.name is None:
            raise SyntaxError('K8sObject: name: [ {0} ] must be set to CREATE the object.'.format(self.name))

        # HTTP 500 : resourceVersion may not be set on objects to be created
        if self.model.metadata.resource_version is not None:
            self.model.metadata.resource_version = None

        url = '{base}'.format(base=self.base_url)
        post_data = self.serialize()
        state = self.request(method='POST', url=url, data=post_data)

        if not state.get('success'):
            status = state.get('status', '')
            state_data = state.get('data', dict())
            reason = state_data['message'] if 'message' in state_data else state_data
            message = 'K8sObject: CREATE failed : HTTP {0} : {1} : {2}'.format(status, reason, post_data)
            if int(status) == 401:
                raise UnauthorizedException(message)
            if int(status) == 404:
                raise NotFoundException(message)
            if int(status) == 409:
                raise AlreadyExistsException(message)
            if int(status) == 422:
                raise UnprocessableEntityException(message)
            raise BadRequestException(message)
        return self

    def update(self):
        if self.name is None:
            raise SyntaxError('K8sObject: name: [ {0} ] must be set to UPDATE the object.'.format(self.name))

        self.model.metadata.strip(self.model.kind)  # strip server-generated metadata before posting updates

        url = '{base}/{name}'.format(base=self.base_url, name=self.name)
        post_data = self.serialize()
        state = self.request(method='PUT', url=url, data=post_data)

        if not state.get('success'):
            status = state.get('status', '')
            reason = state.get('data', dict()).get('message', None)
            message = 'K8sObject: UPDATE failed: HTTP {0} : {1}'.format(status, reason)
            if int(status) == 404:
                raise NotFoundException(message)
            if int(status) == 422:
                raise UnprocessableEntityException(message)
            raise BadRequestException(message)

        return self

    def patch(self):
        if self.name is None:
            raise SyntaxError('K8sObject: name: [ {0} ] must be set to PATCH the object.'.format(self.name))

        self.model.metadata.strip(self.model.kind)  # strip server-generated metadata before posting updates

        url = '{base}/{name}'.format(base=self.base_url, name=self.name)
        post_data = self.serialize()
        state = self.request(method='PATCH', url=url, data=post_data)

        if not state.get('success'):
            status = state.get('status', '')
            reason = state.get('data', dict()).get('message', None)
            message = 'K8sObject: PATCH failed: HTTP {0} : {1}'.format(status, reason)
            if int(status) == 404:
                raise NotFoundException(message)
            if int(status) == 422:
                raise UnprocessableEntityException(message)
            raise BadRequestException(message)

        return self

    def delete(self, cascade=False):
        if self.name is None:
            raise SyntaxError('K8sObject: name: [ {0} ] must be set to DELETE the object.'.format(self.name))

        url = '{base}/{name}'.format(base=self.base_url, name=self.name)

        delete_opts = DeleteOptions()
        delete_opts.orphan_dependents = not cascade

        state = self.request(method='DELETE', url=url, data=delete_opts.serialize())

        if not state.get('success'):
            status = state.get('status', '')
            reason = state.get('data', dict()).get('message', None)
            message = 'K8sObject: DELETE failed: HTTP {0} : {1}'.format(status, reason)
            if int(status) == 404:
                raise NotFoundException(message)
            raise BadRequestException(message)

        if state.get('success'):
            start_time = time.time()
            try:
                while True:
                    time.sleep(0.2)
                    self.get_model()
                    elapsed_time = time.time() - start_time
                    if elapsed_time >= self.DELETE_TIMEOUT_SECONDS:
                        raise TimedOutException("Timed out on DELETE object: [ {0} ]".format(self.name))
            except NotFoundException:
                pass

        return self

    def server_version(self):
        url = '/version'

        state = self.request(method='GET', url=url)

        if not state.get('success'):
            status = state.get('status', '')
            data = state.get('data', dict())

            if isinstance(data, dict) and 'message' in data:
                reason = data.get('message', None)
            else:
                reason = data
            message = 'K8sObject: GET failed: HTTP {0} : {1}'.format(status, reason)

            if status == 401:
                raise UnauthorizedException(message)
            raise BadRequestException(message)

        return state['data']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import re
from os.path import expanduser, isfile

import yaml
from yaml import YAMLError

DEFAULT_KUBECONFIG = "{0}/.kube/config".format(expanduser("~"))
DEFAULT_API_HOST = "localhost:8888"
DEFAULT_API_VERSION = "v1"
DEFAULT_NAMESPACE = "default"

VALID_API_VERSIONS = ["v1"]

VALID_IP_RE = re.compile(
    r'^(http[s]?\:\/\/)?((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)){3})(:[0-9]+)?$')
VALID_HOST_RE = re.compile(r'^(http[s]?\:\/\/)?([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-\.]*[A-Za-z])+(:[0-9]+)?$')


class K8sConfig(object):
    def __init__(self, kubeconfig=DEFAULT_KUBECONFIG, api_host=DEFAULT_API_HOST, auth=None, cert=None,
                 namespace=DEFAULT_NAMESPACE, pull_secret=None, token=None, version=DEFAULT_API_VERSION):
        """
        Pulls configuration from a kubeconfig file, if present, otherwise accepts user-defined parameters.
        See http://kubernetes.io/docs/user-guide/kubeconfig-file/ for information on the kubeconfig file.

        :param kubeconfig: Absolute path to the kubeconfig file, if any.
        :param api_host: Absolute URI where the API server resides.
        :param auth: A tuple of (username, password) for basic authentication.
        :param namespace: The namespace to use. Defaults to 'default'
        :param pull_secret: The password to use when pulling images from the container repository.
        :param token: An authentication token. Mutually exclusive with 'auth'.
        :param version: The version of the API to target. Defaults to 'v1'.
        """

        super(K8sConfig, self).__init__()

        self.api_host = None
        self.auth = None
        self.ca_cert = None
        self.ca_cert_data = None
        self.cert = None
        self.client_certificate = None
        self.client_key = None
        self.pull_secret = None
        self.token = None

        dotconf = None
        if kubeconfig is not None:
            if not isfile(kubeconfig):
                raise IOError('K8sConfig: kubeconfig: [ {0} ] doesn\'t exist.'.format(kubeconfig))
            try:
                with open(kubeconfig, 'r') as stream:
                    dotconf = yaml.load(stream)
            except YAMLError as err:
                raise SyntaxError('K8sConfig: kubeconfig: [ {0} ] is not a valid YAML file.'.format(kubeconfig))

        # we're pulling configuration from a kubeconfig file
        if dotconf is not None:
            self.clusters = dotconf['clusters']
            self.contexts = dotconf['contexts']
            self.current_context = dotconf['current-context']
            self.current_context_dict = [context['context']
                                         for context in self.contexts
                                         if context['name'] == self.current_context][0]
            self.preferences = dotconf['preferences']
            self.pull_secret = pull_secret
            self.users = dotconf['users']
            self.version = dotconf['apiVersion']

            if self.clusters:
                for cluster in self.clusters:
                    if cluster['name'] == self.current_context_dict['cluster']:
                        if 'server' in cluster['cluster']:
                            self.api_host = cluster['cluster']['server']
                        if 'certificate-authority' in cluster['cluster']:
                            self.ca_cert = cluster['cluster']['certificate-authority']
                        if 'certificate-authority-data' in cluster['cluster']:
                            self.ca_cert_data = cluster['cluster']['certificate-authority-data']

            if self.users:
                for user in self.users:
                    if user['name'] == self.current_context_dict['user']:
                        if 'username' in user['user'] and 'password' in user['user']:
                            self.auth = (user['user']['username'], user['user']['password'])
                        if 'token' in user['user']:
                            self.token = user['user']['token']
                        if 'client-certificate' in user['user'] and 'client-key' in user['user']:
                            self.client_certificate = user['user']['client-certificate']
                            self.client_key = user['user']['client-key']
                            self.cert = (self.client_certificate, self.client_key)

            if self.contexts:
                for context in self.contexts:
                    if context['name'] == self.current_context:
                        if 'namespace' in context['context']:
                            self.namespace = context['context']['namespace']
                        else:
                            self.namespace = namespace

        # we're using user-supplied config; run some sanity checks.
        if dotconf is None:

            if not isinstance(api_host, str) or not isinstance(version, str):
                raise SyntaxError(
                    'K8sConfig: host: [ {0} ] and version: [ {1} ] must be strings.'.format(api_host, version))
            if not (VALID_IP_RE.match(api_host) or VALID_HOST_RE.match(api_host)):
                raise SyntaxError('K8sConfig: host: [ {0} ] is invalid.'.format(api_host))
            if auth is not None and not isinstance(auth, tuple):
                raise SyntaxError('K8sConfig: auth: [ {0} ] must be a tuple for basic authentication.'.format(auth))
            if not isinstance(namespace, str):
                raise SyntaxError('K8sConfig: namespace: [ {0} ] must be a string.'.format(namespace))
            if pull_secret is not None and not isinstance(pull_secret, list):
                raise SyntaxError('K8sConfig: pull_secret: [ {0} ] must be a list.'.format(pull_secret))
            if token is not None and not isinstance(token, str):
                raise SyntaxError('K8sConfig: token: [ {0} ] must be a string.'.format(token))
            if version not in VALID_API_VERSIONS:
                valid = ", ".join(VALID_API_VERSIONS)
                raise SyntaxError('K8sConfig: api_version: [ {0} ] must be in: [ {1} ]'.format(version, valid))

            schema_re = re.compile(r"^http[s]*")
            if not schema_re.search(api_host):
                api_host = "http://{0}".format(api_host)
            self.api_host = api_host
            self.auth = auth
            self.cert = cert
            self.namespace = namespace
            self.pull_secret = pull_secret
            self.token = token
            self.version = version

    def serialize(self):
        data = {}
        if self.api_host is not None:
            data['api_host'] = self.api_host
        if self.auth is not None:
            data['auth'] = self.auth
        if self.cert is not None:
            data['cert'] = self.cert
        if self.namespace is not None:
            data['namespace'] = self.namespace
        if self.pull_secret is not None:
            data['pull_secret'] = self.pull_secret
        if self.token is not None:
            data['token'] = self.token
        if self.version is not None:
            data['version'] = self.version
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_list


class ExecAction(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_execaction
    """

    def __init__(self, model=None):
        super(ExecAction, self).__init__()

        self._command = []

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'command' in model:
            self.command = model['command']

    # ------------------------------------------------------------------------------------- command

    @property
    def command(self):
        return self._command

    @command.setter
    def command(self, command=None):
        if not is_valid_list(command, str):
            raise SyntaxError('ExecAction: command: [ {0} ] is invalid.'.format(command))
        self._command = command

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.command is not None:
            data['command'] = self.command
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.NodeSelectorTerm import NodeSelectorTerm


class PreferredSchedulingTerm(object):
    """
    https://kubernetes.io/docs/api-reference/v1.6/#preferredschedulingterm-v1-core
    """

    def __init__(self, model=None):
        super(PreferredSchedulingTerm, self).__init__()

        self._weight = 1  # in the range 1 - 100
        self._preference = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'preference' in model:
            self.preference = NodeSelectorTerm(model['preference'])
        if 'weight' in model:
            self.weight = model['weight']

    # ------------------------------------------------------------------------------------- preference

    @property
    def preference(self):
        return self._preference

    @preference.setter
    def preference(self, p=None):
        if not isinstance(p, NodeSelectorTerm):
            raise SyntaxError('PreferredSchedulingTerm: preference: [ {} ] is invalid.'.format(p))
        self._preference = p

    # ------------------------------------------------------------------------------------- weight

    @property
    def weight(self):
        return self._weight

    @weight.setter
    def weight(self, w=None):
        if not isinstance(w, int):
            raise SyntaxError('PreferredSchedulingTerm: weight: [ {} ] is invalid.'.format(w))
        if not 1 <= w <= 100:
            raise SyntaxError('PreferredSchedulingTerm: weight: [ {} ] must be between 1 and 100.'.format(w))
        self._weight = w

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.preference is not None:
            data['preference'] = self.preference.serialize()
        if self.weight is not None:
            data['weight'] = self.weight
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#


class DaemonSetStatus(object):

    def __init__(self, model=None):
        super(DaemonSetStatus, self).__init__()

        self._current_number_scheduled = None
        self._number_misscheduled = None
        self._desired_number_scheduled = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'currentNumberScheduled' in model:
            self.current_number_scheduled = model['currentNumberScheduled']
        if 'numberMisscheduled' in model:
            self.number_misscheduled = model['numberMisscheduled']
        if 'desiredNumberScheduled' in model:
            self.desired_number_scheduled = model['desiredNumberScheduled']

    # ------------------------------------------------------------------------------------- currentNumberScheduled

    @property
    def current_number_scheduled(self):
        return self._current_number_scheduled

    @current_number_scheduled.setter
    def current_number_scheduled(self, n=None):
        if not isinstance(n, int):
            raise SyntaxError('DaemonSetStatus: current_number_scheduled: [ {} ] is invalid.'.format(n))
        self._current_number_scheduled = n

    # ------------------------------------------------------------------------------------- numberMisscheduled

    @property
    def number_misscheduled(self):
        return self._number_misscheduled

    @number_misscheduled.setter
    def number_misscheduled(self, n=None):
        if not isinstance(n, int):
            raise SyntaxError('DaemonSetStatus: number_misscheduled: [ {} ] is invalid.'.format(n))
        self._number_misscheduled = n

    # ------------------------------------------------------------------------------------- desiredNumberScheduled

    @property
    def desired_number_scheduled(self):
        return self._desired_number_scheduled

    @desired_number_scheduled.setter
    def desired_number_scheduled(self, n=None):
        if not isinstance(n, int):
            raise SyntaxError('DaemonSetStatus: desired_number_scheduled: [ {} ] is invalid.'.format(n))
        self._desired_number_scheduled = n

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.current_number_scheduled is not None:
            data['currentNumberScheduled'] = self.current_number_scheduled
        if self.number_misscheduled is not None:
            data['numberMisscheduled'] = self.number_misscheduled
        if self.desired_number_scheduled is not None:
            data['desiredNumberScheduled'] = self.desired_number_scheduled
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.PersistentVolumeSpec import PersistentVolumeSpec
from kubernetes.models.v1.ResourceRequirements import ResourceRequirements
from kubernetes.models.v1beta1.LabelSelector import LabelSelector
from kubernetes.utils import is_valid_list, is_valid_string


class PersistentVolumeClaimSpec(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_persistentvolumeclaimspec
    """

    VALID_RESOURCES = ['storage']

    def __init__(self, model=None):
        super(PersistentVolumeClaimSpec, self).__init__()

        self._access_modes = []
        self._selector = LabelSelector()
        self._resources = ResourceRequirements()
        self._volume_name = None
        self._storage_class_name = ""

        self.access_modes = ['ReadWriteOnce']
        self.resources.requests = {'storage': '10Gi'}

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'accessModes' in model:
            self.access_modes = model['accessModes']
        if 'resources' in model:
            self.resources = ResourceRequirements(model['resources'])
        if 'storageClassName' in model:
            self.storage_class_name = model['storageClassName']
        if 'selector' in model:
            self.selector = LabelSelector(model['selector'])
        if 'volumeName' in model:
            self.volume_name = model['volumeName']

    # ------------------------------------------------------------------------------------- accessModes

    @property
    def access_modes(self):
        return self._access_modes

    @access_modes.setter
    def access_modes(self, modes=None):
        if not is_valid_list(modes, str):
            raise SyntaxError('PersistentVolumeClaimSpec: access_modes: [ {} ] is invalid.'.format(modes))
        filtered = list(filter(lambda x: x in PersistentVolumeSpec.VALID_ACCESS_MODES, modes))
        self._access_modes = filtered

    # ------------------------------------------------------------------------------------- resources

    @property
    def resources(self):
        return self._resources

    @resources.setter
    def resources(self, res=None):
        if not isinstance(res, ResourceRequirements):
            raise SyntaxError('PersistentVolumeClaimSpec: resources: [ {} ] is invalid.'.format(res))
        self._resources = res

    # ------------------------------------------------------------------------------------- selector

    @property
    def selector(self):
        return self._selector

    @selector.setter
    def selector(self, sel=None):
        if not isinstance(sel, LabelSelector):
            raise SyntaxError('PersistentVolumeClaimSpec: selector: [ {} ] is invalid.'.format(sel))
        self._selector = sel

    # ------------------------------------------------------------------------------------- storage_class_name

    @property
    def storage_class_name(self):
        return self._storage_class_name

    @storage_class_name.setter
    def storage_class_name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('PersistentVolumeClaimSpec: storage_class_name: [ {} ] is invalid.'.format(name))
        self._storage_class_name = name

    # ------------------------------------------------------------------------------------- volumeName

    @property
    def volume_name(self):
        return self._volume_name

    @volume_name.setter
    def volume_name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('PersistentVolumeClaimSpec: volume_name: [ {} ] is invalid.'.format(name))
        self._volume_name = name

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.access_modes is not None:
            data['accessModes'] = self.access_modes
        if self.selector is not None:
            data['selector'] = self.selector.serialize()
        if self.storage_class_name is not None:
            data['storageClassName'] = self.storage_class_name
        if self.resources is not None:
            data['resources'] = self.resources.serialize()
        if self.volume_name is not None:
            data['volumeName'] = self.volume_name
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#
import base64
import json
import os
import tempfile
import requests
from kubernetes.utils.ConvertData import convert
from six.moves.urllib.parse import urlencode


class HttpRequest:
    def __init__(self, method='GET', host='localhost:80', url='/', data=None, auth=None,
                 cert=None, ca_cert=None, ca_cert_data=None, token=None):

        self.http_method = method
        self.http_host = host
        self.url = url
        self.data = data
        self.auth = auth
        self.cert = cert
        self.ca_cert = ca_cert
        self.ca_cert_data = ca_cert_data
        self.token = token

    def send(self):
        state = dict(success=False, reason=None, status=None, data=None)
        http_headers = dict()
        http_headers['Accept'] = 'application/json'

        if self.http_method in ['PUT', 'POST', 'PATCH']:
            http_headers['Content-type'] = 'application/json'

        if self.token is not None:
            http_headers['Authorization'] = 'Bearer {token}'.format(token=self.token)

        if self.data is not None and self.http_method in ['GET']:
            url = "{0}?{1}".format(self.url, urlencode(self.data))
            self.url = url

        self.url = self.http_host + self.url

        temp = None
        verify = False
        if self.ca_cert is not None:
            verify = self.ca_cert
        if self.ca_cert_data is not None:
            temp = tempfile.NamedTemporaryFile(delete=False)
            data = base64.b64decode(self.ca_cert_data)
            temp.write(data)
            temp.close()
            verify = temp.name

        try:

            response = requests.request(
                method=self.http_method,
                url=self.url,
                auth=self.auth,
                cert=self.cert,
                headers=http_headers,
                data="" if self.data is None else json.dumps(self.data),
                verify=verify
            )

        except Exception as err:
            raise err

        finally:
            if temp is not None:
                os.unlink(temp.name)

        state['status'] = response.status_code
        state['reason'] = response.reason
        resp_data = response.content.decode()

        if len(resp_data) > 0:
            try:
                state['data'] = convert(data=json.loads(resp_data))
            except ValueError:
                state['data'] = resp_data.strip()

        if 200 <= state['status'] <= 299:
            state['success'] = True

        return state



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

# from kubernetes.models.v1beta1.CPUTargetUtilization import CPUTargetUtilization
# from kubernetes.models.v1beta1.DaemonSet import DaemonSet
# from kubernetes.models.v1beta1.DaemonSetSpec import DaemonSetSpec
# from kubernetes.models.v1beta1.DaemonSetStatus import DaemonSetStatus
# from kubernetes.models.v1beta1.Deployment import Deployment
# from kubernetes.models.v1beta1.DeploymentRollback import DeploymentRollback
# from kubernetes.models.v1beta1.DeploymentSpec import DeploymentSpec
# from kubernetes.models.v1beta1.DeploymentStatus import DeploymentStatus
# from kubernetes.models.v1beta1.DeploymentStrategy import DeploymentStrategy
# from kubernetes.models.v1beta1.LabelSelector import LabelSelector
# from kubernetes.models.v1beta1.LabelSelectorRequirement import LabelSelectorRequirement
# from kubernetes.models.v1beta1.ReplicaSet import ReplicaSet
# from kubernetes.models.v1beta1.ReplicaSetSpec import ReplicaSetSpec
# from kubernetes.models.v1beta1.ReplicaSetStatus import ReplicaSetStatus
# from kubernetes.models.v1beta1.RollbackConfig import RollbackConfig
# from kubernetes.models.v1beta1.RollingUpdateDeployment import RollingUpdateDeployment
# from kubernetes.models.v1beta1.StatefulSet import StatefulSet
# from kubernetes.models.v1beta1.StatefulSetSpec import StatefulSetSpec
# from kubernetes.models.v1beta1.StatefulSetStatus import StatefulSetStatus
# from kubernetes.models.v1beta1.StorageClass import StorageClass
# from kubernetes.models.v1beta1.SubresourceReference import SubresourceReference



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1.Event import Event
from dateutil.parser import parse


class K8sEvent(K8sObject):

    def __init__(self, config=None, name=None):
        super(K8sEvent, self).__init__(
            config=config,
            name=name,
            obj_type='Event'
        )

    def create(self):
        raise NotImplementedError("K8sEvent: cannot create events this way.")

    def update(self):
        raise NotImplementedError("K8sEvent: cannot update events this way.")

    def list(self, pattern=None, reverse=True):
        ls = super(K8sEvent, self).list()
        events = list(map(lambda x: Event(x), ls))
        if pattern is not None:
            events = list(filter(lambda x: pattern in x.name, events))
        k8s = []
        for x in events:
            j = K8sEvent(config=self.config, name=x.name)
            j.model = x
            k8s.append(j)
        k8s.sort(key=lambda x: x.last_timestamp, reverse=reverse)
        return k8s

    # ------------------------------------------------------------------------------------- warnings

    def warnings(self, pattern=None, reverse=True):
        ls = self.list(pattern, reverse)
        w = filter(lambda x: x.type == "Warning", ls)
        return w

    # ------------------------------------------------------------------------------------- type

    @property
    def type(self):
        return self.model.type

    @type.setter
    def type(self, t=None):
        raise NotImplementedError("K8sEvent: type is read-only.")

    # ------------------------------------------------------------------------------------- count

    @property
    def count(self):
        return self.model.count

    @count.setter
    def count(self, c=None):
        raise NotImplementedError("K8sEvent: count is read-only.")

    # ------------------------------------------------------------------------------------- source

    @property
    def source(self):
        return self.model.source

    @source.setter
    def source(self, s=None):
        raise NotImplementedError("K8sEvent: source is read-only.")

    # ------------------------------------------------------------------------------------- message

    @property
    def message(self):
        return self.model.message.strip()

    @message.setter
    def message(self, m=None):
        raise NotImplementedError("K8sEvent: message is read-only.")

    # ------------------------------------------------------------------------------------- reason

    @property
    def reason(self):
        return self.model.reason

    @reason.setter
    def reason(self, r=None):
        raise NotImplementedError("K8sEvent: reason is read-only.")

    # ------------------------------------------------------------------------------------- involvedObject

    @property
    def involved_object(self):
        return self.model.involved_object

    @involved_object.setter
    def involved_object(self, o=None):
        raise NotImplementedError("K8sEvent: involved_object is read-only.")

    # ------------------------------------------------------------------------------------- firstTimestamp

    @property
    def first_timestamp(self):
        dt = parse(self.model.first_timestamp)
        return dt

    @first_timestamp.setter
    def first_timestamp(self, t=None):
        raise NotImplementedError("K8sEvent: first_timestamp is read_only.")

    # ------------------------------------------------------------------------------------- lastTimestamp

    @property
    def last_timestamp(self):
        dt = parse(self.model.last_timestamp)
        return dt

    @last_timestamp.setter
    def last_timestamp(self, t=None):
        raise NotImplementedError("K8sEvent: last_timestamp is read-only.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import re


class RollingUpdateDeployment(object):
    """
    http://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_rollingupdatedeployment
    """

    def __init__(self, model=None):
        super(RollingUpdateDeployment, self).__init__()

        self._max_unavailable = 1
        self._max_surge = 1

        self.percent_pattern = re.compile(r"^\d+%*$")

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'maxUnavailable' in model:
            self.max_unavailable = model['maxUnavailable']
        if 'maxSurge' in model:
            self.max_surge = model['maxSurge']

    # ------------------------------------------------------------------------------------- maxUnavailable

    @property
    def max_unavailable(self):
        return self._max_unavailable

    @max_unavailable.setter
    def max_unavailable(self, mu=None):
        if not (isinstance(mu, int) or isinstance(mu, str)):
            raise SyntaxError('RollingUpdateDeployment: max_unavailable: [ {} ] is invalid.'.format(mu))
        if isinstance(mu, str) and not self.percent_pattern.match(mu):
            raise SyntaxError('RollingUpdateDeployment: max_unavailable: [ {} ] is invalid.'.format(mu))
        self._max_unavailable = mu

    # ------------------------------------------------------------------------------------- maxSurge

    @property
    def max_surge(self):
        return self._max_surge

    @max_surge.setter
    def max_surge(self, ms=None):
        if not (isinstance(ms, int) or isinstance(ms, str)):
            raise SyntaxError('RollingUpdateDeployment: max_surge: [ {} ] is invalid.'.format(ms))
        if isinstance(ms, str) and not self.percent_pattern.match(ms):
            raise SyntaxError('RollingUpdateDeployment: max_unavailable: [ {} ] is invalid.'.format(ms))
        self._max_surge = ms

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.max_unavailable is not None:
            data['maxUnavailable'] = self.max_unavailable
        if self.max_surge is not None:
            data['maxSurge'] = self.max_surge
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.unversioned.BaseModel import BaseModel
from kubernetes.models.v1.ComponentCondition import ComponentCondition
from kubernetes.models.v1.ObjectMeta import ObjectMeta
from kubernetes.utils import filter_model


class ComponentStatus(BaseModel):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_componentstatus
    """

    def __init__(self, model=None):
        super(ComponentStatus, self).__init__(model=model)

        self._conditions = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        super(ComponentStatus, self).build_with_model(model)

        if 'conditions' in model:
            if not isinstance(model['conditions'], list):
                raise SyntaxError('ComponentStatus: _build_with_model: conditions : [ {0} ] is invalid.'
                                  .format(model['conditions']))
            l = list()
            for c in model['conditions']:
                l.append(ComponentCondition(c))
            self.conditions = l

    # ------------------------------------------------------------------------------------- conditions

    @property
    def conditions(self):
        return self._conditions

    @conditions.setter
    def conditions(self, l):
        if not isinstance(l, list):
            raise SyntaxError('ComponentStatus: conditions: [ {0} ] is invalid.'.format(l))
        self._conditions = l

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = super(ComponentStatus, self).serialize()

        if self.conditions:
            l = list()
            for c in self.conditions:
                l.append(c.serialize())
            data['conditions'] = l
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class PersistentVolumeClaimVolumeSource(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_persistentvolumeclaimvolumesource
    """

    def __init__(self, model=None):
        super(PersistentVolumeClaimVolumeSource, self).__init__()

        self._claim_name = None
        self._read_only = False

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'claimName' in model:
            self.claim_name = model['claimName']
        if 'readOnly' in model:
            self.read_only = model['readOnly']

    # ------------------------------------------------------------------------------------- claimName

    @property
    def claim_name(self):
        return self._claim_name

    @claim_name.setter
    def claim_name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('PersistentVolumeClaimVolumeSource: claim_name: [ {} ] is invalid.'.format(name))
        self._claim_name = name

    # ------------------------------------------------------------------------------------- readOnly

    @property
    def read_only(self):
        return self._read_only

    @read_only.setter
    def read_only(self, ro=None):
        if not isinstance(ro, bool):
            raise SyntaxError('PersistentVolumeClaimVolumeSource: read_only: [ {} ] is invalid.'.format(ro))
        self._read_only = ro

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.claim_name is not None:
            data['claimName'] = self.claim_name
        if self.read_only is not None:
            data['readOnly'] = self.read_only
        return data


#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import filter_model, is_valid_string


class ContainerPort(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_containerport
    """

    VALID_PROTOCOLS = ['TCP', 'UDP']

    def __init__(self, model=None):
        super(ContainerPort, self).__init__()

        self._name = None
        self._host_port = None
        self._container_port = None
        self._protocol = 'TCP'
        self._host_ip = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'name' in model:
            self.name = model['name']
        if 'hostPort' in model:
            self.host_port = model['hostPort']
        if 'containerPort' in model:
            self.container_port = model['containerPort']
        if 'protocol' in model:
            self.protocol = model['protocol']
        if 'hostIP' in model:
            self.host_ip = model['hostIP']

    @staticmethod
    def _is_valid_port(port=None):
        if not isinstance(port, int):
            return False
        if not 0 < port < 65536:
            return False
        return True

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('ContainerPort: name: [ {0} ] is invalid.'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- host port

    @property
    def host_port(self):
        return self._host_port

    @host_port.setter
    def host_port(self, port=None):
        if not ContainerPort._is_valid_port(port):
            raise SyntaxError('ContainerPort: host_port: [ {0} ] is invalid.'.format(port))
        self._host_port = port

    # ------------------------------------------------------------------------------------- container port

    @property
    def container_port(self):
        return self._container_port

    @container_port.setter
    def container_port(self, port=None):
        if not ContainerPort._is_valid_port(port):
            raise SyntaxError('ContainerPort: container_port: [ {0} ] is invalid.'.format(port))
        self._container_port = port

    # ------------------------------------------------------------------------------------- protocol

    @property
    def protocol(self):
        return self._protocol

    @protocol.setter
    def protocol(self, protocol=None):
        if protocol.upper() not in ContainerPort.VALID_PROTOCOLS:
            raise SyntaxError('ContainerPort: protocol: [ {0} ] is invalid.'.format(protocol))
        self._protocol = protocol.upper()

    # ------------------------------------------------------------------------------------- hostIP

    @property
    def host_ip(self):
        return self._host_ip

    @host_ip.setter
    def host_ip(self, ip=None):
        if not is_valid_string(ip):
            raise SyntaxError('ContainerPort: host_ip: [ {0} ] is invalid.'.format(ip))
        self._host_ip = ip

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.name:
            data['name'] = self.name
        if self.host_port:
            data['hostPort'] = self.host_port
        if self.container_port:
            data['containerPort'] = self.container_port
        if self.protocol:
            data['protocol'] = self.protocol
        if self.host_ip:
            data['hostIP'] = self.host_ip
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, filter_model
from kubernetes.models.v1.KeyToPath import KeyToPath


class ConfigMapProjection(object):
    """
    https://kubernetes.io/docs/api-reference/v1.8/#configmapprojection-v1-core

    Adapts a ConfigMap into a projected volume.

    The contents of the target ConfigMap's Data field will be presented in a projected volume as files using the keys
    in the Data field as the file names, unless the items element is populated with specific mappings of keys to paths.
    Note that this is identical to a configmap volume source without the default mode.
    """

    def __init__(self, model=None):
        super(ConfigMapProjection, self).__init__()

        self._items = None
        self._name = None
        self._optional = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'items' in model:
            self.items = model['items']
        if 'name' in model:
            self.name = model['name']
        if 'optional' in model:
            self.optional = model['optional']

    # ------------------------------------------------------------------------------------- items

    @property
    def items(self):
        return self._items

    @items.setter
    def items(self, items=None):
        if not isinstance(items, list):
            raise SyntaxError('ConfigMapVolumeSource: items: [ {0} ] is invalid.'.format(items))
        modeled_items = list()
        for i in items:
            tmp_item = KeyToPath(model=i)
            modeled_items.append(tmp_item)
        self._items = items

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('ConfigMapVolumeSource: name: [ {0} ] is invalid.'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- optional

    @property
    def optional(self):
        return self._optional

    @optional.setter
    def optional(self, v=None):
        if not isinstance(v, bool):
            raise SyntaxError('ConfigMapVolumeSource: optional: [ {0} ] is invalid.'.format(v))
        self._optional = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.items is not None:
            tmp_items = list()
            for i in self.items:
                assert isinstance(i, KeyToPath)
                tmp_items.append(i.serialize())
            data['items'] = tmp_items
        if self.name is not None:
            data['name'] = self.name
        if self.optional is not None:
            data['optional'] = self.optional
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.OwnerReference import OwnerReference
from kubernetes.utils import is_valid_string, filter_model, is_valid_list, is_valid_dict


class ObjectMeta(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_objectmeta
    """

    def __init__(self, model=None):
        super(ObjectMeta, self).__init__()

        self._name = None
        self._generate_name = None
        self._namespace = None
        self._self_link = None
        self._uid = None
        self._resource_version = None
        self._generation = None
        self._creation_timestamp = None
        self._deletion_timestamp = None
        self._deletion_grace_period_seconds = None
        self._labels = {}
        self._annotations = {}
        self._owner_references = None
        self._finalizers = None
        self._cluster_name = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'name' in model:
            self.name = model['name']
        if 'generateName' in model:
            self.generate_name = model['generateName']
        if 'namespace' in model:
            self.namespace = model['namespace']
        if 'selfLink' in model:
            self.self_link = model['selfLink']
        if 'uid' in model:
            self.uid = model['uid']
        if 'resourceVersion' in model:
            self.resource_version = model['resourceVersion']
        if 'generation' in model:
            self.generation = model['generation']
        if 'creationTimestamp' in model:
            self.creation_timestamp = model['creationTimestamp']
        if 'deletionTimestamp' in model:
            self.deletion_timestamp = model['deletionTimestamp']
        if 'deletionGracePeriodSeconds' in model:
            self.deletion_grace_period_seconds = model['deletionGracePeriodSeconds']
        if 'labels' in model:
            self.labels = model['labels']
        if 'annotations' in model:
            self.annotations = model['annotations']
        if 'ownerReferences' in model:
            refs = []
            for o in model['ownerReferences']:
                ref = OwnerReference(o)
                refs.append(ref)
            self.owner_references = refs
        if 'finalizers' in model:
            self.finalizers = model['finalizers']
        if 'clusterName' in model:
            self.cluster_name = model['clusterName']

    # ------------------------------------------------------------------------------------- name

    def strip(self, kind=None):
        # self._name = None
        self._generate_name = None
        # self._namespace = None
        self._self_link = None
        self._uid = None
        if kind != 'Service':
            self._resource_version = None
        self._generation = None
        self._creation_timestamp = None
        self._deletion_timestamp = None
        self._deletion_grace_period_seconds = None
        # self._labels = {}
        # self._annotations = {}
        self._owner_references = None
        self._finalizers = None
        self._cluster_name = None

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('ObjectMeta: name: [ {0} ] is invalid.'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- generate name

    @property
    def generate_name(self):
        return self._generate_name

    @generate_name.setter
    def generate_name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('ObjectMeta: generate_name: [ {0} ] is invalid.'.format(name))
        self._generate_name = name

    # ------------------------------------------------------------------------------------- namespace

    @property
    def namespace(self):
        return self._namespace

    @namespace.setter
    def namespace(self, namespace=None):
        if not is_valid_string(namespace):
            raise SyntaxError('ObjectMeta: namespace: [ {0} ] is invalid.'.format(namespace))
        self._namespace = namespace

    # ------------------------------------------------------------------------------------- selfLink

    @property
    def self_link(self):
        return self._self_link

    @self_link.setter
    def self_link(self, link=None):
        if not is_valid_string(link):
            raise SyntaxError('ObjectMeta: self_link: [ {0} ] is invalid.'.format(link))
        self._self_link = link

    # ------------------------------------------------------------------------------------- uid

    @property
    def uid(self):
        return self._uid

    @uid.setter
    def uid(self, uid=None):
        if uid is not None:
            if not is_valid_string(uid):
                raise SyntaxError('ObjectMeta: uid: [ {0} ] is invalid.'.format(uid))
        self._uid = uid

    # ------------------------------------------------------------------------------------- resourceVersion

    @property
    def resource_version(self):
        return self._resource_version

    @resource_version.setter
    def resource_version(self, v=None):
        if v is not None:
            if not isinstance(v, str):
                raise SyntaxError('ObjectMeta: resource_version: [ {0} ] is invalid.'.format(v))
        self._resource_version = v

    # ------------------------------------------------------------------------------------- generation

    @property
    def generation(self):
        return self._generation

    @generation.setter
    def generation(self, gen=None):
        if not isinstance(gen, int):
            raise SyntaxError('ObjectMeta: generation: [ {0} ] is invalid.'.format(gen))
        self._generation = gen

    # ------------------------------------------------------------------------------------- creationTimestamp

    @property
    def creation_timestamp(self):
        return self._creation_timestamp

    @creation_timestamp.setter
    def creation_timestamp(self, time=None):
        if not is_valid_string(time):
            raise SyntaxError('ObjectMeta: creation_timestamp: [ {0} ] is invalid.'.format(time))
        self._creation_timestamp = time

    # ------------------------------------------------------------------------------------- deletionTimestamp

    @property
    def deletion_timestamp(self):
        return self._deletion_timestamp

    @deletion_timestamp.setter
    def deletion_timestamp(self, time=None):
        if not is_valid_string(time):
            raise SyntaxError('ObjectMeta: deletion_timestamp: [ {0} ] is invalid.'.format(time))
        self._deletion_timestamp = time

    # ------------------------------------------------------------------------------------- deletionGracePeriodSeconds

    @property
    def deletion_grace_period_seconds(self):
        return self._deletion_grace_period_seconds

    @deletion_grace_period_seconds.setter
    def deletion_grace_period_seconds(self, secs=None):
        if not isinstance(secs, int):
            raise SyntaxError('ObjectMeta: deletion_grace_period_seconds: [ {0} ] is invalid.'.format(secs))
        self._deletion_grace_period_seconds = secs

    # ------------------------------------------------------------------------------------- labels

    @property
    def labels(self):
        return self._labels

    @labels.setter
    def labels(self, labels=None):
        if not is_valid_dict(labels, type=str):
            raise SyntaxError('ObjectMeta: labels: [ {0} ] is invalid.'.format(labels))
        self._labels = labels

    # ------------------------------------------------------------------------------------- annotations

    @property
    def annotations(self):
        return self._annotations

    @annotations.setter
    def annotations(self, anns=None):
        if not is_valid_dict(anns, str):
            raise SyntaxError('ObjectMeta: annotations: [ {0} ] is invalid.'.format(anns))
        self._annotations = anns

    # ------------------------------------------------------------------------------------- ownerReferences

    @property
    def owner_references(self):
        return self._owner_references

    @owner_references.setter
    def owner_references(self, refs=None):
        if not is_valid_list(refs, OwnerReference):
            raise SyntaxError('ObjectMeta: owner_references: [ {0} ] is invalid.'.format(refs))
        self._owner_references = refs

    # ------------------------------------------------------------------------------------- finalizers

    @property
    def finalizers(self):
        return self._finalizers

    @finalizers.setter
    def finalizers(self, f=None):
        if not is_valid_list(f, str):
            raise SyntaxError('ObjectMeta: finalizers: [ {0} ] is invalid.'.format(f))
        self._finalizers = f

    # ------------------------------------------------------------------------------------- clusterName

    @property
    def cluster_name(self):
        return self._cluster_name

    @cluster_name.setter
    def cluster_name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('ObjectMeta: cluster_name: [ {0} ] is invalid.'.format(name))
        self._cluster_name = name

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.name:
            data['name'] = self.name
        if self.generate_name:
            data['generateName'] = self.generate_name
        if self.namespace:
            data['namespace'] = self.namespace
        if self.self_link:
            data['selfLink'] = self.self_link
        if self.uid:
            data['uid'] = self.uid
        if self.resource_version:
            data['resourceVersion'] = self.resource_version
        if self.generation:
            data['generation'] = self.generation
        if self.creation_timestamp:
            data['creationTimestamp'] = self.creation_timestamp
        if self.deletion_timestamp:
            data['deletionTimestamp'] = self.deletion_timestamp
        if self.deletion_grace_period_seconds:
            data['deletionGracePeriodSeconds'] = self.deletion_grace_period_seconds
        if self.labels:
            data['labels'] = self.labels
        if self.annotations:
            data['annotations'] = self.annotations
        if self.owner_references:
            data['ownerReferences'] = self.owner_references
        if self.finalizers:
            data['finalizers'] = self.finalizers
        if self.cluster_name:
            data['clusterName'] = self.cluster_name
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class Taint(object):
    """
    https://kubernetes.io/docs/api-reference/v1.6/#taint-v1-core
    """

    VALID_TAINT_EFFECTS = ['NoSchedule', 'PreferNoSchedule', 'NoExecute']

    def __init__(self, model=None):
        super(Taint, self).__init__()

        self._effect = None
        self._key = None
        self._time_added = None
        self._value = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'effect' in model:
            self.effect = model['effect']
        if 'key' in model:
            self.key = model['key']
        if 'timeAdded' in model:
            self.time_added = model['timeAdded']
        if 'value' in model:
            self.value = model['value']

    # ------------------------------------------------------------------------------------- effect

    @property
    def effect(self):
        return self._effect

    @effect.setter
    def effect(self, e=None):
        if not is_valid_string(e) or e not in Taint.VALID_TAINT_EFFECTS:
            raise SyntaxError('Taint: effect: [ {} ] is invalid.'.format(e))
        self._effect = e

    # ------------------------------------------------------------------------------------- key

    @property
    def key(self):
        return self._key

    @key.setter
    def key(self, k=None):
        if not is_valid_string(k):
            raise SyntaxError('Taint: key: [ {} ] is invalid.'.format(k))
        self._key = k

    # ------------------------------------------------------------------------------------- timeAdded

    @property
    def time_added(self):
        return self._time_added

    @time_added.setter
    def time_added(self, t=None):
        # TODO(froch): at time of writing, K8s documentation is incomplete
        # https://kubernetes.io/docs/api-reference/v1.6/#time-v1-meta
        self._time_added = t

    # ------------------------------------------------------------------------------------- value

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, v=None):
        if not is_valid_string(v):
            raise SyntaxError('Taint: value: [ {} ] is invalid.'.format(v))
        self._value = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.effect is not None:
            data['effect'] = self.effect
        if self.key is not None:
            data['key'] = self.key
        if self.time_added is not None:
            data['timeAdded'] = self.time_added
        if self.value is not None:
            data['value'] = self.value
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v2alpha1.CronJob import CronJob


class ScheduledJob(CronJob):
    """
    Making ScheduledJobs a subclass of CronJobs. 
    
    ScheduledJobs exist only in Kubernetes 1.4.x.
    CronJobs exist from Kubernetes 1.5.x onwards.
    """

    def __init__(self, model=None):
        super(ScheduledJob, self).__init__(model=model)
        self.kind = "ScheduledJob"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.PodAffinityTerm import PodAffinityTerm


class WeightedPodAffinityTerm(object):
    """
    https://kubernetes.io/docs/api-reference/v1.6/#weightedpodaffinityterm-v1-core
    """

    def __init__(self, model=None):
        super(WeightedPodAffinityTerm, self).__init__()

        self._pod_affinity_term = None
        self._weight = 1  # in the range 1-100

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'podAffinityTerm' in model:
            self.pod_affinity_term = PodAffinityTerm(model['podAffinityTerm'])
        if 'weight' in model:
            self.weight = model['weight']

    # ------------------------------------------------------------------------------------- podAffinityTerm

    @property
    def pod_affinity_term(self):
        return self._pod_affinity_term

    @pod_affinity_term.setter
    def pod_affinity_term(self, t=None):
        if not isinstance(t, PodAffinityTerm):
            raise SyntaxError('WeightedPodAffinityTerm: pod_affinity_term: [ {} ] is invalid.'.format(t))
        self._pod_affinity_term = t

    # ------------------------------------------------------------------------------------- weight

    @property
    def weight(self):
        return self._weight

    @weight.setter
    def weight(self, w=None):
        if not isinstance(w, int):
            raise SyntaxError('WeightedPodAffinityTerm: weight: [ {} ] is invalid.'.format(w))
        if not 1 <= w <= 100:
            raise SyntaxError('WeightedPodAffinityTerm: weight: [ {} ] must be between 1 and 100.'.format(w))
        self._weight = w

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.pod_affinity_term:
            data['podAffinityTerm'] = self.pod_affinity_term.serialize()
        if self.weight:
            data['weight'] = self.weight
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.EnvVarSource import EnvVarSource
from kubernetes.utils import is_valid_string


class EnvVar(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_envvar
    """

    def __init__(self, model=None):
        super(EnvVar, self).__init__()

        self._name = None
        self._value = None
        self._value_from = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'name' in model:
            self.name = model['name']
        if 'value' in model:
            self.value = model['value']
        if 'valueFrom' in model:
            self.value_from = EnvVarSource(model['valueFrom'])

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, n=None):
        if not is_valid_string(n):
            raise SyntaxError('EnvVar: name: [ {} ] is invalid.'.format(n))
        self._name = n

    # ------------------------------------------------------------------------------------- value

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, v=None):
        if not is_valid_string(v):
            raise SyntaxError('EnvVar: value: [ {} ] is invalid.'.format(v))
        self._value = v

    # ------------------------------------------------------------------------------------- valueFrom

    @property
    def value_from(self):
        return self._value_from

    @value_from.setter
    def value_from(self, v=None):
        if not isinstance(v, EnvVarSource):
            raise SyntaxError('EnvVar: value_from: [ {} ] is invalid.'.format(v))
        self._value_from = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.name is not None:
            data['name'] = self.name
        if self.value is not None:
            data['value'] = self.value
        if self.value_from is not None:
            data['valueFrom'] = self.value_from.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.NodeSelectorTerm import NodeSelectorTerm
from kubernetes.utils import is_valid_list


class NodeSelector(object):
    """
    https://kubernetes.io/docs/api-reference/v1.6/#nodeselector-v1-core
    """

    def __init__(self, model=None):
        super(NodeSelector, self).__init__()

        self._node_selector_terms = []

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'nodeSelectorTerms' in model:
            terms = []
            for t in model['nodeSelectorTerms']:
                term = NodeSelectorTerm(t)
                terms.append(term)
            self.node_selector_terms = terms

    # ------------------------------------------------------------------------------------- nodeSelectorTerms

    @property
    def node_selector_terms(self):
        return self._node_selector_terms

    @node_selector_terms.setter
    def node_selector_terms(self, t=None):
        if not is_valid_list(t, NodeSelectorTerm):
            raise SyntaxError('NodeSelector: node_selector_terms: [ {} ] is invalid.'.format(t))
        self._node_selector_terms = t

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.node_selector_terms:
            terms = []
            for t in self.node_selector_terms:
                term = t.serialize()
                terms.append(term)
            data['nodeSelectorTerms'] = terms
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, filter_model


class GitRepoVolumeSource(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_gitrepovolumesource
    """

    def __init__(self, model=None):
        super(GitRepoVolumeSource, self).__init__()

        self._repository = None
        self._revision = None
        self._directory = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'repository' in model:
            self.repository = model['repository']
        if 'revision' in model:
            self.revision = model['revision']
        if 'directory' in model:
            self.directory = model['directory']

    # ------------------------------------------------------------------------------------- repository

    @property
    def repository(self):
        return self._repository

    @repository.setter
    def repository(self, repo=None):
        if not is_valid_string(repo):
            raise SyntaxError('GitRepoVolumeSource: repository: [ {0} ] is invalid.'.format(repo))
        self._repository = repo

    # ------------------------------------------------------------------------------------- revision

    @property
    def revision(self):
        return self._revision

    @revision.setter
    def revision(self, revision=None):
        if not is_valid_string(revision):
            raise SyntaxError('GitRepoVolumeSource: revision: [ {0} ] is invalid.'.format(revision))
        self._revision = revision

    # ------------------------------------------------------------------------------------- directory

    @property
    def directory(self):
        return self._directory

    @directory.setter
    def directory(self, directory=None):
        if not is_valid_string(directory):
            raise SyntaxError('GitRepoVolumeSource: directory: [ {0} ] is invalid.'.format(directory))
        self._directory = directory

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.repository is not None:
            data['repository'] = self.repository
        if self.revision is not None:
            data['revision'] = self.revision
        if self.directory is not None:
            data['directory'] = self.directory
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.unversioned.BaseModel import BaseModel
from kubernetes.models.v1.ObjectMeta import ObjectMeta
from kubernetes.models.v1beta1.ReplicaSetSpec import ReplicaSetSpec
from kubernetes.models.v1beta1.ReplicaSetStatus import ReplicaSetStatus


class ReplicaSet(BaseModel):
    """
    http://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_replicaset
    """

    def __init__(self, model=None):
        super(ReplicaSet, self).__init__()

        self.kind = 'ReplicaSet'
        self.api_version = 'extensions/v1beta1'

        self.spec = ReplicaSetSpec()
        self.status = ReplicaSetStatus()

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        super(ReplicaSet, self).build_with_model(model)

        if 'spec' in model:
            self.spec = ReplicaSetSpec(model['spec'])
        if 'status' in model:
            self.status = ReplicaSetStatus(model['status'])

    # ------------------------------------------------------------------------------------- spec

    @property
    def spec(self):
        return self._spec

    @spec.setter
    def spec(self, spec=None):
        if not isinstance(spec, ReplicaSetSpec):
            raise SyntaxError('ReplicaSet: spec: [ {} ] is invalid.'.format(spec))
        self._spec = spec

    # ------------------------------------------------------------------------------------- status

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, status=None):
        if not isinstance(status, ReplicaSetStatus):
            raise SyntaxError('ReplicaSet: status: [ {} ] is invalid.'.format(status))
        self._status = status

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = super(ReplicaSet, self).serialize()

        if self.spec is not None:
            data['spec'] = self.spec.serialize()
        if self.status is not None:
            data['status'] = self.status.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.ObjectMeta import ObjectMeta
from kubernetes.utils import is_valid_string


class BaseModel(object):

    def __init__(self, model=None):
        super(BaseModel, self).__init__()

        self._api_version = None
        self._kind = None
        self._metadata = ObjectMeta()
        self._spec = None
        self._status = None

        if model is not None:
            self.build_with_model(model)

    def build_with_model(self, model=None):
        if 'apiVersion' in model:
            self.api_version = model['apiVersion']
        if 'kind' in model:
            self.kind = model['kind']
        if 'metadata' in model:
            self.metadata = ObjectMeta(model['metadata'])

    def __eq__(self, other):
        # see https://github.com/kubernetes/kubernetes/blob/master/docs/design/identifiers.md
        if isinstance(other, self.__class__):
            return self.metadata.namespace == other.metadata.namespace \
                   and self.metadata.name == other.metadata.name
        return NotImplemented

    # ------------------------------------------------------------------------------------- apiVersion

    @property
    def api_version(self):
        return self._api_version

    @api_version.setter
    def api_version(self, v=None):
        if not is_valid_string(v):
            raise SyntaxError('BaseModel: api_version: [ {} ] is invalid.'.format(v))
        self._api_version = v

    # ------------------------------------------------------------------------------------- kind

    @property
    def kind(self):
        return self._kind

    @kind.setter
    def kind(self, k=None):
        if not is_valid_string(k):
            raise SyntaxError('BaseModel: kind: [ {} ] is invalid.'.format(k))
        self._kind = k

    # ------------------------------------------------------------------------------------- metadata

    @property
    def metadata(self):
        return self._metadata

    @metadata.setter
    def metadata(self, meta=None):
        if not isinstance(meta, ObjectMeta):
            raise SyntaxError('BaseModel: metadata: [ {} ] is invalid.'.format(meta))
        self._metadata = meta

    # ------------------------------------------------------------------------------------- spec

    @property
    def spec(self):
        return self._spec

    @spec.setter
    def spec(self, s=None):
        self._spec = s

    # ------------------------------------------------------------------------------------- status

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, s=None):
        self._status = s

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self.metadata.name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('BaseModel: name: [ {} ] is invalid.'.format(name))
        self.metadata.name = name
        self.metadata.labels.update({'name': name})

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.api_version is not None:
            data['apiVersion'] = self.api_version
        if self.kind is not None:
            data['kind'] = self.kind
        if self.metadata is not None:
            data['metadata'] = self.metadata.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.K8sObject import K8sObject
from kubernetes.K8sSecret import K8sSecret
from kubernetes.models.v1.LocalObjectReference import LocalObjectReference
from kubernetes.models.v1.ServiceAccount import ServiceAccount


class K8sServiceAccount(K8sObject):
    def __init__(self, config=None, name=None):

        super(K8sServiceAccount, self).__init__(
            config=config,
            name=name,
            obj_type='ServiceAccount'
        )

    # -------------------------------------------------------------------------------------  override

    def get(self):
        self.model = ServiceAccount(self.get_model())
        return self

    def create(self):
        super(K8sServiceAccount, self).create()
        self.get()
        return self

    def update(self):
        super(K8sServiceAccount, self).update()
        self.get()
        return self

    def list(self, pattern=None):
        ls = super(K8sServiceAccount, self).list()
        accts = list(map(lambda x: ServiceAccount(x), ls))
        if pattern is not None:
            accts = list(filter(lambda x: pattern in x.name, accts))
        k8s = []
        for x in accts:
            j = K8sServiceAccount(config=self.config, name=x.name)
            j.model = x
            k8s.append(j)
        return k8s

    # ------------------------------------------------------------------------------------- add

    def add_api_token(self):
        return K8sSecret.create_service_account_api_token(
            config=self.config,
            name=self.name
        )

    def add_image_pull_secret(self, secret=None):
        if not isinstance(secret, K8sSecret):
            raise SyntaxError('K8sServiceAccount.add_image_pull_secret() secret: [ {} ] is invalid.'.format(secret))
        ref = LocalObjectReference()
        ref.name = secret.name
        refs = self.image_pull_secrets_refs
        refs.append(ref)
        self.image_pull_secrets = refs
        self.update()

    # ------------------------------------------------------------------------------------- secrets

    @property
    def secrets(self):
        secrets = K8sSecret.api_tokens_for_service_account(config=self.config, name=self.name)
        return secrets

    @secrets.setter
    def secrets(self, s=None):
        raise NotImplementedError()

    # ------------------------------------------------------------------------------------- imagePullSecrets

    @property
    def image_pull_secrets_refs(self):
        refs = self.model.image_pull_secrets
        return refs

    @image_pull_secrets_refs.setter
    def image_pull_secrets_refs(self, s=None):
        raise NotImplementedError()

    @property
    def image_pull_secrets(self):
        refs = self.model.image_pull_secrets
        secrets = []
        for ref in refs:
            s = K8sSecret(config=self.config, name=ref.name).get()
            secrets.append(s)
        return secrets

    @image_pull_secrets.setter
    def image_pull_secrets(self, s=None):
        self.model.image_pull_secrets = s



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.LoadBalancerStatus import LoadBalancerStatus


class ServiceStatus(object):

    def __init__(self, model=None):
        super(ServiceStatus, self).__init__()

        self._load_balancer = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'loadBalancer' in model:
            self.load_balancer = LoadBalancerStatus(model['loadBalancer'])

    # ------------------------------------------------------------------------------------- load balancer

    @property
    def load_balancer(self):
        return self._load_balancer

    @load_balancer.setter
    def load_balancer(self, status=None):
        if not isinstance(status, LoadBalancerStatus):
            raise SyntaxError('ServiceStatus: load_balancer: [ {0} ] is invalid.'.format(status))
        self._load_balancer = status

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.load_balancer is not None:
            data['loadBalancer'] = self.load_balancer.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.WeightedPodAffinityTerm import WeightedPodAffinityTerm
from kubernetes.models.v1.PodAffinityTerm import PodAffinityTerm
from kubernetes.utils import is_valid_list


class PodAntiAffinity(object):
    """
    https://kubernetes.io/docs/api-reference/v1.6/#podantiaffinity-v1-core
    https://github.com/kubernetes/community/blob/master/contributors/design-proposals/podaffinity.md
    """

    def __init__(self, model=None):
        super(PodAntiAffinity, self).__init__()

        self._preferred_during_scheduling_ignored_during_execution = []
        self._required_during_scheduling_ignored_during_execution = []

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'preferredDuringSchedulingIgnoredDuringExecution' in model:
            wpats = []
            for x in model['preferredDuringSchedulingIgnoredDuringExecution']:
                wpat = WeightedPodAffinityTerm(x)
                wpats.append(wpat)
            self.preferred_during_scheduling_ignored_during_execution = wpats
        if 'requiredDuringSchedulingIgnoredDuringExecution' in model:
            pats = []
            for x in model['requiredDuringSchedulingIgnoredDuringExecution']:
                pat = PodAffinityTerm(x)
                pats.append(pat)
            self.required_during_scheduling_ignored_during_execution = pats

    # ----------------------------------------------------------------- preferredDuringSchedulingIgnoredDuringExecution

    @property
    def preferred_during_scheduling_ignored_during_execution(self):
        return self._preferred_during_scheduling_ignored_during_execution

    @preferred_during_scheduling_ignored_during_execution.setter
    def preferred_during_scheduling_ignored_during_execution(self, wpats=None):
        if not is_valid_list(wpats, WeightedPodAffinityTerm):
            raise SyntaxError(
                'PodAffinity: preferred_during_scheduling_ignored_during_execution: [ {} ] is invald.'.format(wpats))
        self._preferred_during_scheduling_ignored_during_execution = wpats

    # ----------------------------------------------------------------- requiredDuringSchedulingIgnoredDuringExecution

    @property
    def required_during_scheduling_ignored_during_execution(self):
        return self._required_during_scheduling_ignored_during_execution

    @required_during_scheduling_ignored_during_execution.setter
    def required_during_scheduling_ignored_during_execution(self, pats=None):
        if not is_valid_list(pats, PodAffinityTerm):
            raise SyntaxError(
                'PodAffinity: required_during_scheduling_ignored_during_execution: [ {} ] is invald.'.format(pats))
        self._required_during_scheduling_ignored_during_execution = pats

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.preferred_during_scheduling_ignored_during_execution:
            wpats = []
            for x in self.preferred_during_scheduling_ignored_during_execution:
                wpat = x.serialize()
                wpats.append(wpat)
            data['preferredDuringSchedulingIgnoredDuringExecution'] = wpats
        if self.required_during_scheduling_ignored_during_execution:
            pats = []
            for x in self.required_during_scheduling_ignored_during_execution:
                pat = x.serialize()
                pats.append(pat)
            data['requiredDuringSchedulingIgnoredDuringExecution'] = pats
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#


class TCPSocketAction(object):

    def __init__(self, model=None):
        super(TCPSocketAction, self).__init__()

        self._port = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'port' in model:
            self.port = model['port']

    # ------------------------------------------------------------------------------------- port

    @property
    def port(self):
        return self._port

    @port.setter
    def port(self, port=None):
        msg = 'TCPSocketAction: port: [ {0} ] is invalid.'.format(port)
        if isinstance(port, str) and port.isdigit():
            port = int(port)
        if isinstance(port, int) and not 1 < port < 65535:
            raise SyntaxError(msg)
        self._port = str(port)

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.port:
            data['port'] = self.port
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils.HttpRequest import HttpRequest
from kubernetes.utils.ConvertData import convert
from kubernetes.utils.Helpers import (
    is_valid_dict,
    is_valid_list,
    is_valid_string,
    is_valid_date_time,
    filter_model,
    is_reachable,
    str_to_class,
    is_valid_ip,
    sanitize_version_number
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1beta1.LabelSelectorRequirement import LabelSelectorRequirement
from kubernetes.utils import is_valid_dict, is_valid_list


class LabelSelector(object):
    """
    http://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_labelselector
    """

    def __init__(self, model=None):
        super(LabelSelector, self).__init__()

        self._match_labels = None
        self._match_expressions = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'matchLabels' in model:
            self.match_labels = model['matchLabels']
        if 'matchExpressions' in model:
            exps = []
            for me in model['matchExpressions']:
                exp = LabelSelectorRequirement(me)
                exps.append(exp)
            self.match_expressions = exps

    # ------------------------------------------------------------------------------------- matchLabels

    @property
    def match_labels(self):
        return self._match_labels

    @match_labels.setter
    def match_labels(self, ml=None):
        if not is_valid_dict(ml):
            raise SyntaxError(
                'LabelSelector: match_labels: [ {} ] is invalid.'.format(ml))

        self._match_labels = ml

    # ------------------------------------------------------------------------------------- matchExpressions

    @property
    def match_expressions(self):
        return self._match_expressions

    @match_expressions.setter
    def match_expressions(self, me=None):
        if not is_valid_list(me, LabelSelectorRequirement):
            raise SyntaxError(
                'LabelSelector: match_expressions: [ {} ] is invalid.'.format(me))

        self._match_expressions = me

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.match_labels is not None:
            data['matchLabels'] = self.match_labels
        if self.match_expressions is not None:
            data['matchExpressions'] = [x.serialize() for x in self.match_expressions]

        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.JobCondition import JobCondition
from kubernetes.utils import is_valid_list, is_valid_string


class JobStatus(object):
    """
    http://kubernetes.io/docs/api-reference/batch/v1/definitions/#_v1_jobstatus
    """

    def __init__(self, model=None):
        super(JobStatus, self).__init__()

        self._conditions = []
        self._start_time = None
        self._completion_time = None
        self._active = None
        self._succeeded = None
        self._failed = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'conditions' in model:
            conds = []
            for c in model['conditions']:
                cond = JobCondition(c)
                conds.append(cond)
            self.conditions = conds
        if 'startTime' in model:
            self.start_time = model['startTime']
        if 'completionTime' in model:
            self.completion_time = model['completionTime']
        if 'active' in model:
            self.active = model['active']
        if 'succeeded' in model:
            self.succeeded = model['succeeded']
        if 'failed' in model:
            self.failed = model['failed']

    # --------------------------------------------------------------------------------- conditions

    @property
    def conditions(self):
        return self._conditions

    @conditions.setter
    def conditions(self, conds=None):
        if not is_valid_list(conds, JobCondition):
            raise SyntaxError('JobStatus: conditions: [ {} ] is invalid.'.format(conds))
        self._conditions = conds

    # --------------------------------------------------------------------------------- startTime

    @property
    def start_time(self):
        return self._start_time

    @start_time.setter
    def start_time(self, time=None):
        if not is_valid_string(time):
            raise SyntaxError('JobStatus: start_time: [ {} ] is invalid.'.format(time))
        self._start_time = time

    # --------------------------------------------------------------------------------- completionTime

    @property
    def completion_time(self):
        return self._completion_time

    @completion_time.setter
    def completion_time(self, time=None):
        if not is_valid_string(time):
            raise SyntaxError('JobStatus: completion_time: [ {} ] is invalid.'.format(time))
        self._completion_time = time

    # --------------------------------------------------------------------------------- active

    @property
    def active(self):
        return self._active

    @active.setter
    def active(self, a=None):
        if not isinstance(a, int):
            raise SyntaxError('JobStatus: active: [ {} ] is invalid.'.format(a))
        self._active = a

    # --------------------------------------------------------------------------------- succeeded

    @property
    def succeeded(self):
        return self._succeeded

    @succeeded.setter
    def succeeded(self, s=None):
        if not isinstance(s, int):
            raise SyntaxError('JobStatus: succeeded: [ {} ] is invalid.'.format(s))
        self._succeeded = s

    # --------------------------------------------------------------------------------- failed

    @property
    def failed(self):
        return self._failed

    @failed.setter
    def failed(self, f=None):
        if not isinstance(f, int):
            raise SyntaxError('JobStatus: failed: [ {} ] is invalid.'.format(f))
        self._failed = f

    # --------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.conditions is not None:
            data['conditions'] = [x.serialize() for x in self.conditions]
        if self.start_time is not None:
            data['startTime'] = self.start_time
        if self.completion_time is not None:
            data['completionTime'] = self.completion_time
        if self.active is not None:
            data['active'] = self.active
        if self.succeeded is not None:
            data['succeeded'] = self.succeeded
        if self.failed is not None:
            data['failed'] = self.failed
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.PodTemplateSpec import PodTemplateSpec
from kubernetes.models.v1beta1.DeploymentStrategy import DeploymentStrategy
from kubernetes.models.v1beta1.LabelSelector import LabelSelector
from kubernetes.models.v1beta1.RollbackConfig import RollbackConfig


class DeploymentSpec(object):
    """
    http://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_deploymentspec
    """

    def __init__(self, model=None):
        super(DeploymentSpec, self).__init__()

        self._replicas = 1
        self._selector = LabelSelector()
        self._template = PodTemplateSpec()
        self._strategy = DeploymentStrategy()
        self._min_ready_seconds = 0
        self._revision_history_limit = None
        self._paused = False
        self._rollback_to = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'replicas' in model:
            self.replicas = model['replicas']
        if 'selector' in model:
            self.selector = LabelSelector(model['selector'])
        if 'template' in model:
            self.template = PodTemplateSpec(model['template'])
        if 'strategy' in model:
            self.strategy = DeploymentStrategy(model['strategy'])
        if 'minReadySeconds' in model:
            self.min_ready_seconds = model['minReadySeconds']
        if 'revisionHistoryLimit' in model:
            self.revision_history_limit = model['revisionHistoryLimit']
        if 'paused' in model:
            self.paused = model['paused']
        if 'rollbackTo' in model:
            self.rollback_to = model['rollbackTo']

    # ------------------------------------------------------------------------------------- replicas

    @property
    def replicas(self):
        return self._replicas

    @replicas.setter
    def replicas(self, reps=None):
        if not isinstance(reps, int):
            raise SyntaxError('DeploymentSpec: replicas: [ {} ] is invalid'.format(reps))
        self._replicas = reps

    # ------------------------------------------------------------------------------------- selector

    @property
    def selector(self):
        return self._selector

    @selector.setter
    def selector(self, sel=None):
        if not isinstance(sel, LabelSelector):
            raise SyntaxError('DeploymentSpec: selector: [ {} ] is invalid'.format(sel))
        self._selector = sel

    # ------------------------------------------------------------------------------------- template

    @property
    def template(self):
        return self._template

    @template.setter
    def template(self, t=None):
        if not isinstance(t, PodTemplateSpec):
            raise SyntaxError('DeploymentSpec: template: [ {} ] is invalid'.format(t))
        self._template = t

    # ------------------------------------------------------------------------------------- strategy

    @property
    def strategy(self):
        return self._strategy

    @strategy.setter
    def strategy(self, strat=None):
        if not isinstance(strat, DeploymentStrategy):
            raise SyntaxError('DeploymentSpec: strategy: [ {} ] is invalid'.format(strat))
        self._strategy = strat

    # ------------------------------------------------------------------------------------- minReadySeconds

    @property
    def min_ready_seconds(self):
        return self._min_ready_seconds

    @min_ready_seconds.setter
    def min_ready_seconds(self, mrs=None):
        if not isinstance(mrs, int):
            raise SyntaxError('DeploymentSpec: min_ready_seconds: [ {} ] is invalid'.format(mrs))
        self._min_ready_seconds = mrs

    # ------------------------------------------------------------------------------------- revisionHistoryLimit

    @property
    def revision_history_limit(self):
        return self._revision_history_limit

    @revision_history_limit.setter
    def revision_history_limit(self, rhl=None):
        if not isinstance(rhl, int):
            raise SyntaxError('DeploymentSpec: revision_history_limit: [ {} ] is invalid'.format(rhl))
        self._revision_history_limit = rhl

    # ------------------------------------------------------------------------------------- paused

    @property
    def paused(self):
        return self._paused

    @paused.setter
    def paused(self, p=None):
        if not isinstance(p, bool):
            raise SyntaxError('DeploymentSpec: paused: [ {} ] is invalid'.format(p))
        self._paused = p

    # ------------------------------------------------------------------------------------- rollbackTo

    @property
    def rollback_to(self):
        return self._rollback_to

    @rollback_to.setter
    def rollback_to(self, rc=None):
        if not isinstance(rc, RollbackConfig):
            raise SyntaxError('DeploymentSpec: rollback_to: [ {} ] is invalid'.format(rc))
        self._rollback_to = rc

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.replicas is not None:
            data['replicas'] = self.replicas
        if self.selector is not None:
            data['selector'] = self.selector.serialize()
        if self.template is not None:
            data['template'] = self.template.serialize()
        if self.strategy is not None:
            data['strategy'] = self.strategy.serialize()
        if self.min_ready_seconds is not None:
            data['minReadySeconds'] = self.min_ready_seconds
        if self.revision_history_limit is not None:
            data['revisionHistoryLimit'] = self.revision_history_limit
        if self.paused is not None:
            data['paused'] = self.paused
        if self.rollback_to is not None:
            data['rollbackTo'] = self.rollback_to.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.NodeSelectorRequirement import NodeSelectorRequirement
from kubernetes.utils import is_valid_list


class NodeSelectorTerm(object):
    """
    https://kubernetes.io/docs/api-reference/v1.6/#nodeselectorterm-v1-core
    """

    def __init__(self, model=None):
        super(NodeSelectorTerm, self).__init__()

        self._match_expressions = []

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'matchExpressions' in model:
            exps = []
            for e in model['matchExpressions']:
                exp = NodeSelectorRequirement(e)
                exps.append(exp)
            self.match_expressions = exps

    # ------------------------------------------------------------------------------------- matchExpressions

    @property
    def match_expressions(self):
        return self._match_expressions

    @match_expressions.setter
    def match_expressions(self, e=None):
        if not is_valid_list(e, NodeSelectorRequirement):
            raise SyntaxError('NodeSelectorTerm: match_expressions: [ {} ] is invalid.'.format(e))
        self._match_expressions = e

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.match_expressions:
            reqs = []
            for r in self.match_expressions:
                req = r.serialize()
                reqs.append(req)
            data['matchExpressions'] = reqs
        return data
